#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

import iso8601

import ujson

from gnocchiclient import utils
from gnocchiclient.v1 import base


class AggregatesManager(base.Manager):
    def fetch(self, operations, search=None,
              resource_type='generic', start=None, stop=None, granularity=None,
              needed_overlap=None, groupby=None, fill=None, details=False):
        """Get measurements of an aggregated metrics.

        :param operations: operations
        :type operations: list or str
        :param start: beginning of the period
        :type start: timestamp
        :param stop: end of the period
        :type stop: timestamp
        :param granularity: granularity to retrieve (in seconds)
        :type granularity: int
        :param needed_overlap: percent of datapoints in each metrics required
        :type needed_overlap: float
        :param groupby: list of attribute to group by
        :type groupby: list
        :param fill: value to use when backfilling missing datapoints
        :type fill: float or 'null'
        :param details: also returns the list of metrics or resources
                        associated to the operations
        :type details: boolean

        See Gnocchi REST API documentation for the format
        of *query dictionary*
        http://docs.openstack.org/developer/gnocchi/rest.html#aggregates
        """
        if isinstance(start, datetime.datetime):
            start = start.isoformat()
        if isinstance(stop, datetime.datetime):
            stop = stop.isoformat()

        params = dict(start=start, stop=stop,
                      granularity=granularity,
                      needed_overlap=needed_overlap,
                      fill=fill,
                      details=details)

        data = dict(operations=operations)
        if search is not None:
            data["search"] = search
            data["resource_type"] = resource_type
            params["groupby"] = groupby

        aggregates = self._post("v1/aggregates?%s" % (
            utils.dict_to_querystring(params)),
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(data)).json()

        if search is not None and groupby is not None:
            for group in aggregates:
                self._convert_dates(group["measures"]["measures"])
        else:
            self._convert_dates(aggregates["measures"])
        return aggregates

    @classmethod
    def _convert_dates(cls, data):
        # NOTE(sileht): browse to aggregates measures dict tree and convert
        # date when we found timeseries, dict can looks like
        # {"aggregated": ...}, {"metric_id": {"agg": ...}} or
        # {"resource_id": {"metric_name": {"agg": ...}}}
        for key in data:
            if isinstance(data[key], list):
                data[key] = [(iso8601.parse_date(ts), g, value)
                             for ts, g, value in data[key]]
            elif isinstance(data[key], dict):
                cls._convert_dates(data[key])
            else:
                raise RuntimeError("Unexpected aggregates API output %s" %
                                   data[key])



# -*- encoding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from dateutil import tz

import iso8601

import six
from six.moves.urllib import parse as urllib_parse


def add_query_argument(cmd, parser, *args, **kwargs):
    return parser.add_argument(
        cmd,
        help=u"A query to filter resource. "
        u"The syntax is a combination of attribute, operator and value. "
        u"For example: id=90d58eea-70d7-4294-a49a-170dcdf44c3c would filter "
        u"resource with a certain id. More complex queries can be built, "
        u"e.g.: not (flavor_id!=\"1\" and memory>=24). "
        u"Use \"\" to force data to be interpreted as string. "
        u"Supported operators are: not, and, ∧ or, ∨, >=, <=, !=, >, <, =, "
        u"==, eq, ne, lt, gt, ge, le, in, like, ≠, ≥, ≤, like, in.",
        *args, **kwargs)


def list2cols(cols, objs):
    return cols, [tuple([o[k] for k in cols])
                  for o in objs]


def format_string_list(l):
    return ", ".join(l)


def format_dict_list(l):
    return "\n".join(
        "- " + ", ".join("%s: %s" % (k, v)
                         for k, v in elem.items())
        for elem in l)


def format_dict_dict(value):
    return "\n".join(
        "- %s: " % name + " , ".join("%s: %s" % (k, v)
                                     for k, v in elem.items())
        for name, elem in value.items())


def format_move_dict_to_root(obj, field):
    for attr in obj[field]:
        obj["%s/%s" % (field, attr)] = obj[field][attr]
    del obj[field]


def format_resource_type(rt):
    format_move_dict_to_root(rt, "attributes")
    for key in rt:
        if key.startswith("attributes"):
            rt[key] = ", ".join(
                "%s=%s" % (k, v) for k, v in sorted(rt[key].items()))


def format_archive_policy(ap):
    ap['definition'] = format_dict_list(ap['definition'])
    ap['aggregation_methods'] = format_string_list(ap['aggregation_methods'])


def format_resource_for_metric(metric):
    # NOTE(sileht): Gnocchi < 2.0
    if 'resource' not in metric:
        return

    if not metric['resource']:
        metric['resource/id'] = None
        del metric['resource']
    else:
        format_move_dict_to_root(metric, "resource")


def dict_from_parsed_args(parsed_args, attrs):
    d = {}
    for attr in attrs:
        value = getattr(parsed_args, attr)
        if value is not None:
            d[attr] = value
    return d


def dict_to_querystring(objs):
    strings = []
    for k, values in sorted(objs.items()):
        if values is not None:
            if not isinstance(values, (list, tuple)):
                values = [values]
            strings.append("&".join(
                ("%s=%s" % (k, v)
                 for v in map(urllib_parse.quote,
                              map(six.text_type, values)))))
    return "&".join(strings)


def get_pagination_options(parsed_args):
    options = dict(
        sorts=parsed_args.sort,
        limit=parsed_args.limit,
        marker=parsed_args.marker)

    if hasattr(parsed_args, 'details'):
        options['details'] = parsed_args.details
    if hasattr(parsed_args, 'history'):
        options['history'] = parsed_args.history
    return options


def build_pagination_options(details=False, history=False,
                             limit=None, marker=None, sorts=None):
    options = {}
    if details:
        options["details"] = "true"
    if history:
        options["history"] = "true"
    if limit:
        options["limit"] = int(limit)
    if marker:
        options["marker"] = marker
    if sorts:
        options["sort"] = sorts
    return options


def get_client(obj):
    if hasattr(obj.app, 'client_manager'):
        # NOTE(sileht): cliff objects loaded by OSC
        return obj.app.client_manager.metric
    else:
        # TODO(sileht): Remove this when OSC is able
        # to install the gnocchi client binary itself
        return obj.app.client


LOCAL_TIMEZONE = tz.gettz()


def parse_date(s):
    """Parse date from string.

    If no timezone is specified, default is assumed to be local time zone.

    :param s: The date to parse.
    :type s: str
    """
    return iso8601.parse_date(s, LOCAL_TIMEZONE)


def dt_to_localtz(d):
    return d.astimezone(LOCAL_TIMEZONE)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cliff import show

from gnocchiclient import utils


class CliStatusShow(show.ShowOne):
    """Show the status of measurements processing."""

    def take_action(self, parsed_args):
        status = utils.get_client(self).status.get()

        d = {
            "storage/total number of measures to process":
            status['storage']['summary']['measures'],
            "storage/number of metric having measures to process":
            status['storage']['summary']['metrics'],
        }

        if 'metricd' in status:
            d["metricd/processors"] = status['metricd']['processors']

        return self.dict2columns(d)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
import sys
import warnings

from cliff import app
from cliff import commandmanager

from keystoneauth1 import adapter
from keystoneauth1 import exceptions
from keystoneauth1 import loading

from gnocchiclient import auth
from gnocchiclient import benchmark
from gnocchiclient import client
from gnocchiclient.v1 import aggregates_cli
from gnocchiclient.v1 import archive_policy_cli
from gnocchiclient.v1 import archive_policy_rule_cli as ap_rule_cli
from gnocchiclient.v1 import build_cli
from gnocchiclient.v1 import capabilities_cli
from gnocchiclient.v1 import metric_cli
from gnocchiclient.v1 import resource_cli
from gnocchiclient.v1 import resource_type_cli
from gnocchiclient.v1 import status_cli
from gnocchiclient.version import __version__


class GnocchiCommandManager(commandmanager.CommandManager):
    SHELL_COMMANDS = {
        "status": status_cli.CliStatusShow,
        "server version": build_cli.CliBuildShow,
        "resource list": resource_cli.CliResourceList,
        "resource show": resource_cli.CliResourceShow,
        "resource history": resource_cli.CliResourceHistory,
        "resource search": resource_cli.CliResourceSearch,
        "resource create": resource_cli.CliResourceCreate,
        "resource update": resource_cli.CliResourceUpdate,
        "resource delete": resource_cli.CliResourceDelete,
        "resource batch delete": resource_cli.CliResourceBatchDelete,
        "resource-type list": resource_type_cli.CliResourceTypeList,
        "resource-type create": resource_type_cli.CliResourceTypeCreate,
        "resource-type update": resource_type_cli.CliResourceTypeUpdate,
        "resource-type show": resource_type_cli.CliResourceTypeShow,
        "resource-type delete": resource_type_cli.CliResourceTypeDelete,
        "archive-policy list": archive_policy_cli.CliArchivePolicyList,
        "archive-policy show": archive_policy_cli.CliArchivePolicyShow,
        "archive-policy create": archive_policy_cli.CliArchivePolicyCreate,
        "archive-policy update": archive_policy_cli.CliArchivePolicyUpdate,
        "archive-policy delete": archive_policy_cli.CliArchivePolicyDelete,
        "archive-policy-rule list": ap_rule_cli.CliArchivePolicyRuleList,
        "archive-policy-rule show": ap_rule_cli.CliArchivePolicyRuleShow,
        "archive-policy-rule create": ap_rule_cli.CliArchivePolicyRuleCreate,
        "archive-policy-rule update": ap_rule_cli.CliArchivePolicyRuleUpdate,
        "archive-policy-rule delete": ap_rule_cli.CliArchivePolicyRuleDelete,
        "metric list": metric_cli.CliMetricList,
        "metric show": metric_cli.CliMetricShow,
        "metric create": metric_cli.CliMetricCreate,
        "metric delete": metric_cli.CliMetricDelete,
        "measures show": metric_cli.CliMeasuresShow,
        "measures add": metric_cli.CliMeasuresAdd,
        "measures batch-metrics": metric_cli.CliMetricsMeasuresBatch,
        "measures batch-resources-metrics":
            metric_cli.CliResourcesMetricsMeasuresBatch,
        "measures aggregation": metric_cli.CliMeasuresAggregation,
        "aggregates": aggregates_cli.CliAggregates,
        "capabilities list": capabilities_cli.CliCapabilitiesList,
        "benchmark metric create": benchmark.CliBenchmarkMetricCreate,
        "benchmark metric show": benchmark.CliBenchmarkMetricShow,
        "benchmark measures add": benchmark.CliBenchmarkMeasuresAdd,
        "benchmark measures show": benchmark.CliBenchmarkMeasuresShow,
    }

    def load_commands(self, namespace):
        for name, command_class in self.SHELL_COMMANDS.items():
            self.add_command(name, command_class)


class GnocchiShell(app.App):
    def __init__(self):
        super(GnocchiShell, self).__init__(
            description='Gnocchi command line client',
            version=__version__,
            command_manager=GnocchiCommandManager('gnocchiclient'),
            deferred_help=True,
        )

        self._client = None

    def build_option_parser(self, description, version):
        """Return an argparse option parser for this application.

        Subclasses may override this method to extend
        the parser with more global options.

        :param description: full description of the application
        :paramtype description: str
        :param version: version number for the application
        :paramtype version: str
        """
        parser = super(GnocchiShell, self).build_option_parser(
            description,
            version,
            argparse_kwargs={'allow_abbrev': False})
        parser.add_argument(
            '--gnocchi-api-version',
            default=os.environ.get('GNOCCHI_API_VERSION', '1'),
            help='Defaults to env[GNOCCHI_API_VERSION] or 1.')

        # NOTE(jd) This is a workaroun for people using Keystone auth with the
        # CLI. A lot of rc files do not export OS_AUTH_TYPE=password and
        # assumes it is the default. It's not in that case, but since we can't
        # fix all the rc files of the world, workaround it here.
        if ("OS_PASSWORD" in os.environ and
           "OS_AUTH_TYPE" not in os.environ):
            os.environ.set("OS_AUTH_TYPE", "password")

        loading.register_session_argparse_arguments(parser=parser)
        plugin = loading.register_auth_argparse_arguments(
            parser=parser, argv=sys.argv, default="gnocchi-basic")

        if not isinstance(plugin, (auth.GnocchiNoAuthLoader,
                                   auth.GnocchiBasicLoader)):
            adapter.register_adapter_argparse_arguments(
                parser=parser, service_type="metric")
            adapter.register_service_adapter_argparse_arguments(
                parser=parser, service_type="metric")

            parser.add_argument(
                '--endpoint',
                default=os.environ.get('GNOCCHI_ENDPOINT'),
                help='Gnocchi endpoint (Env: GNOCCHI_ENDPOINT). '
                'Deprecated, use --os-endpoint-override and '
                'OS_ENDPOINT_OVERRIDE instead')

        return parser

    @property
    def client(self):
        # NOTE(sileht): we lazy load the client to not
        # load/connect auth stuffs
        if self._client is None:
            auth_plugin = loading.load_auth_from_argparse_arguments(
                self.options)
            session = loading.load_session_from_argparse_arguments(
                self.options, auth=auth_plugin)

            if isinstance(auth_plugin, (auth.GnocchiNoAuthPlugin,
                                        auth.GnocchiBasicPlugin)):
                # Normal endpoint
                kwargs = dict(
                    version=self.options.gnocchi_api_version,
                    session=session,
                )
            else:
                # Openstck style endpoint
                kwargs = dict(
                    version=(self.options.os_metric_api_version or
                             self.options.os_api_version or
                             self.options.gnocchi_api_version),
                    session=session,
                    adapter_options=dict(
                        service_type=(self.options.os_metric_service_type or
                                      self.options.os_service_type),
                        service_name=(self.options.os_metric_service_name or
                                      self.options.os_service_name),
                        interface=(self.options.os_metric_interface or
                                   self.options.os_interface),
                        region_name=self.options.os_region_name,
                        endpoint_override=(
                            self.options.os_metric_endpoint_override or
                            self.options.os_endpoint_override or
                            self.options.endpoint),
                    )
                )
            self._client = client.Client(**kwargs)
        return self._client

    def clean_up(self, cmd, result, err):
        if err and isinstance(err, exceptions.HttpError):
            try:
                error = err.response.json()
            except Exception:
                pass
            else:
                if 'description' in error:
                    # Gnocchi format
                    print(error['description'])
                elif 'error' in error and 'message' in error['error']:
                    # Keystone format
                    print(error['error']['message'])

    def configure_logging(self):
        if self.options.debug:
            # --debug forces verbose_level 3
            # Set this here so cliff.app.configure_logging() can work
            self.options.verbose_level = 3

        super(GnocchiShell, self).configure_logging()
        root_logger = logging.getLogger('')

        # Set logging to the requested level
        if self.options.verbose_level == 0:
            # --quiet
            root_logger.setLevel(logging.ERROR)
            warnings.simplefilter("ignore")
        elif self.options.verbose_level == 1:
            # This is the default case, no --debug, --verbose or --quiet
            root_logger.setLevel(logging.WARNING)
            warnings.simplefilter("ignore")
        elif self.options.verbose_level == 2:
            # One --verbose
            root_logger.setLevel(logging.INFO)
            warnings.simplefilter("once")
        elif self.options.verbose_level >= 3:
            # Two or more --verbose
            root_logger.setLevel(logging.DEBUG)

        # Hide some useless message
        requests_log = logging.getLogger("requests")
        cliff_log = logging.getLogger('cliff')
        stevedore_log = logging.getLogger('stevedore')
        iso8601_log = logging.getLogger("iso8601")

        cliff_log.setLevel(logging.ERROR)
        stevedore_log.setLevel(logging.ERROR)
        iso8601_log.setLevel(logging.ERROR)

        if self.options.debug:
            requests_log.setLevel(logging.DEBUG)
        else:
            requests_log.setLevel(logging.ERROR)


def main(args=None):
    if args is None:
        args = sys.argv[1:]
    return GnocchiShell().run(args)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

import six


class ClientException(Exception):
    """The base exception class for all exceptions this library raises."""

    message = 'Unknown Error'

    def __init__(self, code=None, message=None, request_id=None,
                 url=None, method=None):
        self.code = code
        self.message = message or self.__class__.message
        self.request_id = request_id
        self.url = url
        self.method = method

    def __str__(self):
        formatted_string = "%s" % self.message
        if self.code:
            formatted_string += " (HTTP %s)" % self.code
        if self.request_id:
            formatted_string += " (Request-ID: %s)" % self.request_id
        return formatted_string


class RetryAfterException(ClientException):
    """The base exception for ClientExceptions that use Retry-After header."""

    def __init__(self, *args, **kwargs):
        try:
            self.retry_after = int(kwargs.pop('retry_after'))
        except (KeyError, ValueError):
            self.retry_after = 0

        super(RetryAfterException, self).__init__(*args, **kwargs)


class ConnectionFailure(ClientException):
    """Connection failure."""


class ConnectionTimeout(ClientException):
    """Connection timeout."""


class UnknownConnectionError(ClientException):
    """Unknown connection error."""


class SSLError(ClientException):
    """SSL connection error."""


class BadRequest(ClientException):
    """HTTP 400 - Bad request: you sent some malformed data."""

    http_status = 400
    message = "Bad request"


class Unauthorized(ClientException):
    """HTTP 401 - Unauthorized: bad credentials."""

    http_status = 401
    message = "Unauthorized"


class Forbidden(ClientException):
    """HTTP 403 - Forbidden.

    Your credentials don't give you access to this resource.
    """

    http_status = 403
    message = "Forbidden"


class NotFound(ClientException):
    """HTTP 404 - Not found."""

    http_status = 404
    message = "Not found"


class MetricNotFound(NotFound):
    message = "Metric not found"
    match = re.compile("Metric .* does not exist")


class ResourceNotFound(NotFound):
    message = "Resource not found"
    match = re.compile("Resource .* does not exist")


class ResourceTypeNotFound(NotFound):
    message = "Resource type not found"
    match = re.compile("Resource type .* does not exist")


class ArchivePolicyNotFound(NotFound):
    message = "Archive policy not found"
    match = re.compile("Archive policy .* does not exist")


class ArchivePolicyRuleNotFound(NotFound):
    message = "Archive policy rule not found"
    match = re.compile("Archive policy rule .* does not exist")


class MethodNotAllowed(ClientException):
    """HTTP 405 - Method Not Allowed."""

    http_status = 405
    message = "Method Not Allowed"


class NotAcceptable(ClientException):
    """HTTP 406 - Not Acceptable."""

    http_status = 406
    message = "Not Acceptable"


class Conflict(ClientException):
    """HTTP 409 - Conflict."""

    http_status = 409
    message = "Conflict"


class NamedMetricAlreadyExists(Conflict):
    message = "Named metric already exists"
    match = re.compile("Named metric .* already exist")


class ResourceTypeAlreadyExists(Conflict):
    message = "Resource type already exists"
    match = re.compile("Resource type .* already exists")


class ResourceAlreadyExists(Conflict):
    message = "Resource already exists"
    match = re.compile("Resource .* already exists")


class ArchivePolicyAlreadyExists(Conflict):
    message = "Archive policy already exists"
    match = re.compile("Archive policy .* already exists")


class ArchivePolicyRuleAlreadyExists(Conflict):
    message = "Archive policy rule already exists"
    match = re.compile("Archive policy rule .* already exists")


class OverLimit(RetryAfterException):
    """HTTP 413 - Over limit.

    You're over the API limits for this time period.
    """

    http_status = 413
    message = "Over limit"


class RateLimit(RetryAfterException):
    """HTTP 429 - Rate limit.

    You've sent too many requests for this time period.
    """

    http_status = 429
    message = "Rate limit"


class NotImplemented(ClientException):  # noqa
    """HTTP 501 - Not Implemented.

    The server does not support this operation.
    """

    http_status = 501
    message = "Not Implemented"


_error_classes = [BadRequest, Unauthorized, Forbidden, NotFound,
                  MethodNotAllowed, NotAcceptable, Conflict, OverLimit,
                  RateLimit, NotImplemented]
_error_classes_enhanced = {
    NotFound: [MetricNotFound, ResourceTypeNotFound, ResourceNotFound,
               ArchivePolicyRuleNotFound, ArchivePolicyNotFound],
    Conflict: [NamedMetricAlreadyExists, ResourceTypeAlreadyExists,
               ResourceAlreadyExists,
               ArchivePolicyAlreadyExists,
               ArchivePolicyRuleAlreadyExists]
}
_code_map = dict(
    (c.http_status, (c, _error_classes_enhanced.get(c, [])))
    for c in _error_classes)


def from_response(response, method=None):
    """Return an instance of one of the ClientException on an requests response.

    Usage::

        resp, body = requests.request(...)
        if resp.status_code != 200:
            raise from_response(resp)
    """
    if response.status_code:
        cls, enhanced_classes = _code_map.get(response.status_code,
                                              (ClientException, []))

    req_id = response.headers.get("x-openstack-request-id")
    content_type = response.headers.get("Content-Type", "").split(";")[0]

    kwargs = {
        'code': response.status_code,
        'method': method,
        'url': response.url,
        'request_id': req_id,
    }

    if "retry-after" in response.headers:
        kwargs['retry_after'] = response.headers.get('retry-after')

    if content_type == "application/json":
        try:
            body = response.json()
        except ValueError:
            pass
        else:
            if 'description' in body:
                # Gnocchi json
                desc = body.get('description')
                if desc and isinstance(desc, six.text_type):
                    for enhanced_cls in enhanced_classes:
                        if enhanced_cls.match.match(desc):
                            cls = enhanced_cls
                            break
                kwargs['message'] = desc
            elif isinstance(body, dict) and isinstance(body.get("error"),
                                                       dict):
                # Keystone json
                kwargs['message'] = body["error"]["message"]
            else:
                kwargs['message'] = response.text
    elif content_type.startswith("text/"):
        kwargs['message'] = response.text

    if not kwargs['message']:
        del kwargs['message']
    return cls(**kwargs)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ujson

from gnocchiclient import utils
from gnocchiclient.v1 import base


class ResourceManager(base.Manager):
    url = "v1/resource/"

    def list(self, resource_type="generic", details=False, history=False,
             limit=None, marker=None, sorts=None):
        """List resources.

        :param resource_type: Type of the resource
        :type resource_type: str
        :param details: Show all attributes of resources
        :type details: bool
        :param history: Show the history of resources
        :type history: bool
        :param limit: maximum number of resources to return
        :type limit: int
        :param marker: the last item of the previous page; we return the next
                       results after this value.
        :type marker: str
        :param sorts: list of resource attributes to order by. (example
                      ["user_id:desc-nullslast", "project_id:asc"]
        :type sorts: list of str
        """
        params = utils.build_pagination_options(
            details, history, limit, marker, sorts)
        url = "%s%s?%s" % (self.url, resource_type,
                           utils.dict_to_querystring(params))
        return self._get(url).json()

    def get(self, resource_type, resource_id, history=False):
        """Get a resource.

        :param resource_type: Type of the resource
        :type resource_type: str
        :param resource_id: ID of the resource
        :type resource_id: str
        :param history: Show the history of the resource
        :type history: bool
        """
        history = "/history" if history else ""
        url = self.url + "%s/%s%s" % (resource_type, resource_id, history)
        return self._get(url).json()

    def history(self, resource_type, resource_id, details=False,
                limit=None, marker=None, sorts=None):
        """Get a resource.

        :param resource_type: Type of the resource
        :type resource_type: str
        :param resource_id: ID of the resource
        :type resource_id: str
        :param details: Show all attributes of resources
        :type details: bool
        :param limit: maximum number of resources to return
        :type limit: int
        :param marker: the last item of the previous page; we returns the next
                       results after this value.
        :type marker: str
        :param sorts: list of resource attributes to order by. (example
                      ["user_id:desc-nullslast", "project_id:asc"]
        :type sorts: list of str
        """
        params = utils.build_pagination_options(details, False, limit, marker,
                                                sorts)
        url = "%s%s/%s/history?%s" % (self.url, resource_type, resource_id,
                                      utils.dict_to_querystring(params))
        return self._get(url).json()

    def create(self, resource_type, resource):
        """Create a resource.

        :param resource_type: Type of the resource
        :type resource_type: str
        :param resource: Attribute of the resource
        :type resource: dict
        """
        return self._post(
            self.url + resource_type,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(resource)).json()

    def update(self, resource_type, resource_id, resource):
        """Update a resource.

        :param resource_type: Type of the resource
        :type resource_type: str
        :param resource_id: ID of the resource
        :type resource_id: str
        :param resource: Attribute of the resource
        :type resource: dict
        """
        return self._patch(
            self.url + resource_type + "/" + resource_id,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(resource)).json()

    def delete(self, resource_id):
        """Delete a resource.

        :param resource_id: ID of the resource
        :type resource_id: str
        """
        self._delete(self.url + "generic/" + resource_id)

    def batch_delete(self, query, resource_type="generic"):
        """Delete a batch of resources based on attribute values.

        :param resource_type: Type of the resource
        :type resource_type: str
        """
        if isinstance(query, dict):
            return self._delete(
                self.url + resource_type + "/",
                headers={'Content-Type': "application/json"},
                data=ujson.dumps(query)).json()
        return self._delete(
            self.url + resource_type + "/?filter=" + query,
            headers={'Content-Type': "application/json"}).json()

    def search(self, resource_type="generic", query=None, details=False,
               history=False, limit=None, marker=None, sorts=None):
        """List resources.

        :param resource_type: Type of the resource
        :type resource_type: str
        :param query: The query dictionary
        :type query: dict
        :param details: Show all attributes of resources
        :type details: bool
        :param history: Show the history of resources
        :type history: bool
        :param limit: maximum number of resources to return
        :type limit: int
        :param marker: the last item of the previous page; we returns the next
                       results after this value.
        :type marker: str
        :param sorts: list of resource attributes to order by. (example
                      ["user_id:desc-nullslast", "project_id:asc"]
        :type sorts: list of str

        See Gnocchi REST API documentation for the format
        of *query dictionary*
        http://gnocchi.xyz/rest.html#searching-for-resources
        """
        query = query or {}
        params = utils.build_pagination_options(
            details, history, limit, marker, sorts)
        url = "v1/search/resource/%s?%%s" % resource_type

        if isinstance(query, dict):
            return self._post(
                url % utils.dict_to_querystring(params),
                headers={'Content-Type': "application/json"},
                data=ujson.dumps(query)).json()

        params['filter'] = query
        return self._post(
            url % utils.dict_to_querystring(params),
            headers={'Content-Type': "application/json"}).json()



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cliff import command
from cliff import lister
from cliff import show

from gnocchiclient import utils


class CliArchivePolicyList(lister.Lister):
    """List archive policies."""

    COLS = ('name',
            'back_window', 'definition', 'aggregation_methods')

    def take_action(self, parsed_args):
        policies = utils.get_client(self).archive_policy.list()
        if parsed_args.formatter == 'table':
            for ap in policies:
                utils.format_archive_policy(ap)
        return utils.list2cols(self.COLS, policies)


class CliArchivePolicyShow(show.ShowOne):
    """Show an archive policy."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyShow, self).get_parser(prog_name)
        parser.add_argument("name",
                            help="Name of the archive policy")
        return parser

    def take_action(self, parsed_args):
        ap = utils.get_client(self).archive_policy.get(
            name=parsed_args.name)
        if parsed_args.formatter == 'table':
            utils.format_archive_policy(ap)
        return self.dict2columns(ap)


def archive_policy_definition(string):
    parts = string.split(",")
    defs = {}
    for part in parts:
        attr, __, value = part.partition(":")
        if (attr not in ['granularity', 'points', 'timespan'] or
           value is None):
            raise ValueError
        defs[attr] = value
    if len(defs) < 2:
        raise ValueError
    return defs


class CliArchivePolicyWriteBase(show.ShowOne):
    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyWriteBase, self).get_parser(prog_name)
        parser.add_argument("name", help="name of the archive policy")
        parser.add_argument("-d", "--definition", action='append',
                            required=True, type=archive_policy_definition,
                            metavar="<DEFINITION>",
                            help=("two attributes (separated by ',') of an "
                                  "archive policy definition with its name "
                                  "and value separated with a ':'"))
        return parser


class CliArchivePolicyCreate(CliArchivePolicyWriteBase):
    """Create an archive policy."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyCreate, self).get_parser(prog_name)
        parser.add_argument("-b", "--back-window", dest="back_window",
                            type=int,
                            help="back window of the archive policy")
        parser.add_argument("-m", "--aggregation-method",
                            action="append",
                            dest="aggregation_methods",
                            help="aggregation method of the archive policy")
        return parser

    def take_action(self, parsed_args):
        archive_policy = utils.dict_from_parsed_args(
            parsed_args, ['name', 'back_window', 'aggregation_methods',
                          'definition'])
        ap = utils.get_client(self).archive_policy.create(
            archive_policy=archive_policy)
        if parsed_args.formatter == 'table':
            utils.format_archive_policy(ap)
        return self.dict2columns(ap)


class CliArchivePolicyUpdate(CliArchivePolicyWriteBase):
    """Update an archive policy."""

    def take_action(self, parsed_args):
        archive_policy = utils.dict_from_parsed_args(
            parsed_args, ['definition'])
        ap = utils.get_client(self).archive_policy.update(
            name=parsed_args.name, archive_policy=archive_policy)
        if parsed_args.formatter == 'table':
            utils.format_archive_policy(ap)
        return self.dict2columns(ap)


class CliArchivePolicyDelete(command.Command):
    """Delete an archive policy."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyDelete, self).get_parser(prog_name)
        parser.add_argument("name",
                            help="Name of the archive policy")
        return parser

    def take_action(self, parsed_args):
        utils.get_client(self).archive_policy.delete(name=parsed_args.name)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import sys

from cliff import command
from cliff import lister
from cliff import show

import iso8601

from gnocchiclient import utils


LOG_DEP = logging.getLogger('deprecated')


class CliMetricWithResourceID(command.Command):
    def get_parser(self, prog_name):
        parser = super(CliMetricWithResourceID, self).get_parser(prog_name)
        parser.add_argument("--resource-id", "-r",
                            help="ID of the resource")
        return parser


class CliMetricList(lister.Lister):
    """List metrics."""

    COLS = ('id', 'archive_policy/name', 'name', 'unit', 'resource_id')

    def get_parser(self, prog_name):
        parser = super(CliMetricList, self).get_parser(prog_name)
        parser.add_argument("--limit", type=int, metavar="<LIMIT>",
                            help="Number of metrics to return "
                            "(Default is server default)")
        parser.add_argument("--marker", metavar="<MARKER>",
                            help="Last item of the previous listing. "
                            "Return the next results after this value")
        parser.add_argument("--sort", action="append", metavar="<SORT>",
                            help="Sort of metric attribute "
                            "(example: user_id:desc-nullslast")
        return parser

    def take_action(self, parsed_args):
        metrics = utils.get_client(self).metric.list(
            **utils.get_pagination_options(parsed_args))
        for metric in metrics:
            utils.format_archive_policy(metric["archive_policy"])
            utils.format_move_dict_to_root(metric, "archive_policy")
        return utils.list2cols(self.COLS, metrics)


class DeprecatedCliMetricList(CliMetricList):
    """Deprecated: List metrics."""

    def take_action(self, parsed_args):
        LOG_DEP.warning('This command has been deprecated. '
                        'Please use "metric list" instead.')
        return super(DeprecatedCliMetricList, self).take_action(parsed_args)


class CliMetricShow(CliMetricWithResourceID, show.ShowOne):
    """Show a metric."""

    def get_parser(self, prog_name):
        parser = super(CliMetricShow, self).get_parser(prog_name)
        parser.add_argument("metric",
                            help="ID or name of the metric")
        return parser

    def take_action(self, parsed_args):
        metric = utils.get_client(self).metric.get(
            metric=parsed_args.metric,
            resource_id=parsed_args.resource_id)
        metric['archive_policy/name'] = metric["archive_policy"]["name"]
        del metric['archive_policy']
        del metric['created_by_user_id']
        del metric['created_by_project_id']
        utils.format_resource_for_metric(metric)
        return self.dict2columns(metric)


class DeprecatedCliMetricShow(CliMetricShow):
    """Deprecated: Show a metric."""

    def take_action(self, parsed_args):
        LOG_DEP.warning('This command has been deprecated. '
                        'Please use "metric show" instead.')
        return super(DeprecatedCliMetricShow, self).take_action(parsed_args)


class CliMetricCreateBase(show.ShowOne, CliMetricWithResourceID):
    def get_parser(self, prog_name):
        parser = super(CliMetricCreateBase, self).get_parser(prog_name)
        parser.add_argument("--archive-policy-name", "-a",
                            dest="archive_policy_name",
                            help="name of the archive policy")
        return parser


class CliMetricCreate(CliMetricCreateBase):
    """Create a metric."""

    def get_parser(self, prog_name):
        parser = super(CliMetricCreate, self).get_parser(prog_name)
        parser.add_argument("name", nargs='?',
                            metavar="METRIC_NAME",
                            help="Name of the metric")
        parser.add_argument("--unit", "-u",
                            help="unit of the metric")
        return parser

    def take_action(self, parsed_args):
        metric = utils.get_client(self).metric._create_new(
            archive_policy_name=parsed_args.archive_policy_name,
            name=parsed_args.name,
            resource_id=parsed_args.resource_id,
            unit=parsed_args.unit,
        )
        utils.format_resource_for_metric(metric)
        if 'archive_policy' in metric:
            metric['archive_policy/name'] = metric["archive_policy"]["name"]
            del metric['archive_policy']
        del metric['created_by_user_id']
        del metric['created_by_project_id']
        return self.dict2columns(metric)


class DeprecatedCliMetricCreate(CliMetricCreate):
    """Deprecated: Create a metric."""

    def take_action(self, parsed_args):
        LOG_DEP.warning('This command has been deprecated. '
                        'Please use "metric create" instead.')
        return super(DeprecatedCliMetricCreate, self).take_action(parsed_args)


class CliMetricDelete(CliMetricWithResourceID):
    """Delete a metric."""

    def get_parser(self, prog_name):
        parser = super(CliMetricDelete, self).get_parser(prog_name)
        parser.add_argument("metric", nargs='+',
                            help="IDs or names of the metric")
        return parser

    def take_action(self, parsed_args):
        for metric in parsed_args.metric:
            utils.get_client(self).metric.delete(
                metric=metric, resource_id=parsed_args.resource_id)


class DeprecatedCliMetricDelete(CliMetricDelete):
    """Deprecated: Delete a metric."""

    def take_action(self, parsed_args):
        LOG_DEP.warning('This command has been deprecated. '
                        'Please use "metric delete" instead.')
        return super(DeprecatedCliMetricDelete, self).take_action(parsed_args)


class CliMeasuresReturn(lister.Lister):
    def get_parser(self, prog_name):
        parser = super(CliMeasuresReturn, self).get_parser(prog_name)
        parser.add_argument("--utc", help="Return timestamps as UTC",
                            default=False,
                            action="store_true")
        return parser

    @staticmethod
    def format_measures_with_tz(parsed_args, measures):
        if parsed_args.utc:
            def t(x):
                return x
        else:
            t = utils.dt_to_localtz
        return [(t(dt).isoformat(), g, v) for dt, g, v in measures]


class CliMeasuresShow(CliMetricWithResourceID, CliMeasuresReturn,
                      lister.Lister):
    """Get measurements of a metric."""

    COLS = ('timestamp', 'granularity', 'value')

    def get_parser(self, prog_name):
        parser = super(CliMeasuresShow, self).get_parser(prog_name)
        parser.add_argument("metric",
                            help="ID or name of the metric")
        parser.add_argument("--aggregation",
                            help="aggregation to retrieve")
        parser.add_argument("--start",
                            type=utils.parse_date,
                            help="beginning of the period")
        parser.add_argument("--stop",
                            type=utils.parse_date,
                            help="end of the period")
        parser.add_argument("--granularity",
                            help="granularity to retrieve")
        parser.add_argument("--refresh", action="store_true",
                            help="force aggregation of all known measures")
        parser.add_argument("--resample",
                            help=("granularity to resample time-series to "
                                  "(in seconds)"))
        return parser

    def take_action(self, parsed_args):
        measures = utils.get_client(self).metric.get_measures(
            metric=parsed_args.metric,
            resource_id=parsed_args.resource_id,
            aggregation=parsed_args.aggregation,
            start=parsed_args.start,
            stop=parsed_args.stop,
            granularity=parsed_args.granularity,
            refresh=parsed_args.refresh,
            resample=parsed_args.resample
        )
        return self.COLS, self.format_measures_with_tz(parsed_args, measures)


class CliMeasuresAddBase(CliMetricWithResourceID):
    def get_parser(self, prog_name):
        parser = super(CliMeasuresAddBase, self).get_parser(prog_name)
        parser.add_argument("metric", help="ID or name of the metric")
        return parser


class CliMeasuresAdd(CliMeasuresAddBase):
    """Add measurements to a metric."""

    def measure(self, measure):
        timestamp, __, value = measure.rpartition("@")
        try:
            timestamp = utils.parse_date(timestamp).isoformat()
        except iso8601.iso8601.ParseError:
            # NOTE(sileht): return string as-is and let the server decide
            # if it's valid. (like +2hour, now, -5day)
            pass
        return {'timestamp': timestamp, 'value': float(value)}

    def get_parser(self, prog_name):
        parser = super(CliMeasuresAdd, self).get_parser(prog_name)
        parser.add_argument("-m", "--measure", action='append',
                            required=True, type=self.measure,
                            help=("timestamp and value of a measure "
                                  "separated with a '@'"))
        return parser

    def take_action(self, parsed_args):
        utils.get_client(self).metric.add_measures(
            metric=parsed_args.metric,
            resource_id=parsed_args.resource_id,
            measures=parsed_args.measure,
        )


class CliMeasuresBatch(command.Command):
    def stdin_or_file(self, value):
        if value == "-":
            return sys.stdin
        else:
            return open(value, 'r')

    def get_parser(self, prog_name):
        parser = super(CliMeasuresBatch, self).get_parser(prog_name)
        parser.add_argument("file", type=self.stdin_or_file,
                            help=("File containing measurements to batch or "
                                  "- for stdin (see Gnocchi REST API docs for "
                                  "the format"))
        return parser


class CliMetricsMeasuresBatch(CliMeasuresBatch):
    def take_action(self, parsed_args):
        with parsed_args.file as f:
            utils.get_client(self).metric.batch_metrics_measures(json.load(f))


class CliResourcesMetricsMeasuresBatch(CliMeasuresBatch):
    def get_parser(self, prog_name):
        parser = super(CliResourcesMetricsMeasuresBatch, self).get_parser(
            prog_name)
        parser.add_argument("--create-metrics", action='store_true',
                            help="Create unknown metrics"),
        return parser

    def take_action(self, parsed_args):
        with parsed_args.file as f:
            utils.get_client(self).metric.batch_resources_metrics_measures(
                json.load(f), create_metrics=parsed_args.create_metrics)


class CliMeasuresAggregation(CliMeasuresReturn):
    """Get measurements of aggregated metrics."""

    COLS = ('timestamp', 'granularity', 'value')

    def get_parser(self, prog_name):
        parser = super(CliMeasuresAggregation, self).get_parser(prog_name)
        parser.add_argument("-m", "--metric", nargs='+', required=True,
                            help="metrics IDs or metric name")
        parser.add_argument("--aggregation", help="granularity aggregation "
                                                  "function to retrieve")
        parser.add_argument("--reaggregation",
                            help="groupby aggregation function to retrieve")
        parser.add_argument("--start",
                            type=utils.parse_date,
                            help="beginning of the period")
        parser.add_argument("--stop",
                            type=utils.parse_date,
                            help="end of the period")
        parser.add_argument("--granularity",
                            help="granularity to retrieve")
        parser.add_argument("--needed-overlap", type=float,
                            help=("percent of datapoints in each "
                                  "metrics required"))
        utils.add_query_argument("--query", parser)
        parser.add_argument("--resource-type", default="generic",
                            help="Resource type to query"),
        parser.add_argument("--groupby",
                            action='append',
                            help="Attribute to use to group resources"),
        parser.add_argument("--refresh", action="store_true",
                            help="force aggregation of all known measures")
        parser.add_argument("--resample",
                            help=("granularity to resample time-series to "
                                  "(in seconds)"))
        parser.add_argument("--fill",
                            help=("Value to use when backfilling timestamps "
                                  "with missing values in a subset of series. "
                                  "Value should be a float or 'null'."))
        return parser

    def take_action(self, parsed_args):
        metrics = parsed_args.metric
        if parsed_args.query:
            if len(parsed_args.metric) != 1:
                raise ValueError("One metric is required if query is provided")
            metrics = parsed_args.metric[0]
        measures = utils.get_client(self).metric.aggregation(
            metrics=metrics,
            query=parsed_args.query,
            aggregation=parsed_args.aggregation,
            reaggregation=parsed_args.reaggregation,
            start=parsed_args.start,
            stop=parsed_args.stop,
            granularity=parsed_args.granularity,
            needed_overlap=parsed_args.needed_overlap,
            resource_type=parsed_args.resource_type,
            groupby=parsed_args.groupby,
            refresh=parsed_args.refresh,
            resample=parsed_args.resample, fill=parsed_args.fill
        )
        if parsed_args.groupby:
            ms = []
            for g in measures:
                group_name = ", ".join("%s: %s" % (k, g['group'][k])
                                       for k in sorted(g['group']))
                for m in g['measures']:
                    i = [group_name]
                    i.extend(self.format_measures_with_tz(parsed_args, [m])[0])
                    ms.append(i)
            return ('group',) + self.COLS, ms
        return self.COLS, self.format_measures_with_tz(parsed_args, measures)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import uuid

from gnocchiclient.tests.functional import base


class ResourceTypeClientTest(base.ClientTestBase):
    RESOURCE_TYPE = str(uuid.uuid4())
    RESOURCE_ID = str(uuid.uuid4())

    def test_help(self):
        self.gnocchi("help", params="resource list", has_output=False)

    def test_resource_type_scenario(self):
        # LIST
        result = self.gnocchi('resource-type', params="list")
        r = json.loads(result)
        self.assertEqual([{'attributes': '', 'name': 'generic'}], r)

        # CREATE
        result = self.gnocchi(
            u'resource-type',
            params=u"create -a foo:string:true:max_length=16 "
                   "-a bar:number:no:max=32 %s" % self.RESOURCE_TYPE)
        resource = json.loads(result)
        self.assertEqual(self.RESOURCE_TYPE, resource["name"])
        self.assertEqual(
            "max_length=16, min_length=0, required=True, type=string",
            resource["attributes/foo"])

        # SHOW
        result = self.gnocchi(
            u'resource-type', params=u"show %s" % self.RESOURCE_TYPE)
        resource = json.loads(result)
        self.assertEqual(self.RESOURCE_TYPE, resource["name"])
        self.assertEqual(
            "max_length=16, min_length=0, required=True, type=string",
            resource["attributes/foo"])

        # PATCH
        result = self.gnocchi(
            u'resource-type',
            params=u"update -r foo "
            "-a new:number:yes:max=16:fill=8 %s" % self.RESOURCE_TYPE)
        resource = json.loads(result)
        self.assertEqual(self.RESOURCE_TYPE, resource["name"])
        self.assertNotIn("attributes/foo", resource)
        self.assertEqual(
            "max=16, min=None, required=True, type=number",
            resource["attributes/new"])

        # SHOW
        result = self.gnocchi(
            u'resource-type', params=u"show %s" % self.RESOURCE_TYPE)
        resource = json.loads(result)
        self.assertEqual(self.RESOURCE_TYPE, resource["name"])
        self.assertNotIn("attributes/foo", resource)
        self.assertEqual(
            "max=16, min=None, required=True, type=number",
            resource["attributes/new"])

        # Create a resource for this type
        result = self.gnocchi(
            u'resource', params=(u"create %s -t %s -a new:5") %
            (self.RESOURCE_ID, self.RESOURCE_TYPE))
        resource = json.loads(result)
        self.assertEqual(self.RESOURCE_ID, resource["id"])
        self.assertEqual(5.0, resource["new"])

        # Delete the resource
        self.gnocchi('resource', params="delete %s" % self.RESOURCE_ID,
                     has_output=False)

        # DELETE
        self.gnocchi('resource-type',
                     params="delete %s" % self.RESOURCE_TYPE,
                     has_output=False)

        # DELETE AGAIN
        result = self.gnocchi('resource-type',
                              params="delete %s" % self.RESOURCE_TYPE,
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Resource type %s does not exist (HTTP 404)\n"
            % self.RESOURCE_TYPE,
            result)

        # SHOW AGAIN
        result = self.gnocchi(u'resource-type',
                              params=u"show %s" % self.RESOURCE_TYPE,
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Resource type %s does not exist (HTTP 404)\n"
            % self.RESOURCE_TYPE,
            result)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import distutils.util

from cliff import command
from cliff import lister
from cliff import show

from gnocchiclient import exceptions
from gnocchiclient import utils


class CliResourceList(lister.Lister):
    """List resources."""

    COLS = ('id', 'type',
            'project_id', 'user_id',
            'original_resource_id',
            'started_at', 'ended_at',
            'revision_start', 'revision_end')

    def get_parser(self, prog_name, history=True):
        parser = super(CliResourceList, self).get_parser(prog_name)
        parser.add_argument("--details", action='store_true',
                            help="Show all attributes of generic resources"),
        if history:
            parser.add_argument("--history", action='store_true',
                                help="Show history of the resources"),
        parser.add_argument("--limit", type=int, metavar="<LIMIT>",
                            help="Number of resources to return "
                            "(Default is server default)")
        parser.add_argument("--marker", metavar="<MARKER>",
                            help="Last item of the previous listing. "
                            "Return the next results after this value")
        parser.add_argument("--sort", action="append", metavar="<SORT>",
                            help="Sort of resource attribute "
                            "(example: user_id:desc-nullslast")
        parser.add_argument("--type", "-t", dest="resource_type",
                            default="generic", help="Type of resource")
        return parser

    def _list2cols(self, resources):
        """Return a formatted list of resources."""
        if not resources:
            return self.COLS, []
        cols = list(self.COLS)
        for k in resources[0]:
            if k not in cols:
                cols.append(k)
        if 'creator' in cols:
            cols.remove('created_by_user_id')
            cols.remove('created_by_project_id')
        return utils.list2cols(cols, resources)

    def take_action(self, parsed_args):
        resources = utils.get_client(self).resource.list(
            resource_type=parsed_args.resource_type,
            **utils.get_pagination_options(parsed_args))
        # Do not dump metrics because it makes the list way too long
        for r in resources:
            del r['metrics']
        return self._list2cols(resources)


class CliResourceHistory(CliResourceList):
    """Show the history of a resource."""

    def get_parser(self, prog_name):
        parser = super(CliResourceHistory, self).get_parser(prog_name,
                                                            history=False)
        parser.add_argument("resource_id",
                            help="ID of a resource")
        return parser

    def take_action(self, parsed_args):
        resources = utils.get_client(self).resource.history(
            resource_type=parsed_args.resource_type,
            resource_id=parsed_args.resource_id,
            **utils.get_pagination_options(parsed_args))
        if parsed_args.formatter == 'table':
            return self._list2cols(list(map(normalize_metrics, resources)))
        return self._list2cols(resources)


class CliResourceSearch(CliResourceList):
    """Search resources with specified query rules."""

    def get_parser(self, prog_name):
        parser = super(CliResourceSearch, self).get_parser(prog_name)
        utils.add_query_argument("query", parser)
        return parser

    def take_action(self, parsed_args):
        resources = utils.get_client(self).resource.search(
            resource_type=parsed_args.resource_type,
            query=parsed_args.query,
            **utils.get_pagination_options(parsed_args))
        # Do not dump metrics because it makes the list way too long
        for r in resources:
            del r['metrics']
        return self._list2cols(resources)


def normalize_metrics(res):
    res['metrics'] = "\n".join(sorted(
        ["%s: %s" % (name, _id)
            for name, _id in res['metrics'].items()]))
    return res


class CliResourceShow(show.ShowOne):
    """Show a resource."""

    def get_parser(self, prog_name):
        parser = super(CliResourceShow, self).get_parser(prog_name)
        parser.add_argument("--type", "-t", dest="resource_type",
                            default="generic", help="Type of resource")
        parser.add_argument("resource_id",
                            help="ID of a resource")
        return parser

    def take_action(self, parsed_args):
        res = utils.get_client(self).resource.get(
            resource_type=parsed_args.resource_type,
            resource_id=parsed_args.resource_id)
        if parsed_args.formatter == 'table':
            normalize_metrics(res)
        return self.dict2columns(res)


class CliResourceCreate(show.ShowOne):
    """Create a resource."""

    def get_parser(self, prog_name):
        parser = super(CliResourceCreate, self).get_parser(prog_name)
        parser.add_argument("--type", "-t", dest="resource_type",
                            default="generic", help="Type of resource")
        parser.add_argument("resource_id",
                            help="ID of the resource")
        parser.add_argument("-a", "--attribute", action='append',
                            default=[],
                            help=("name and value of an attribute "
                                  "separated with a ':'"))
        parser.add_argument("-m", "--add-metric", action='append',
                            default=[],
                            help="name:id of a metric to add"),
        parser.add_argument(
            "-n", "--create-metric", action='append', default=[],
            help="name:archive_policy_name of a metric to create"),
        return parser

    def _resource_from_args(self, parsed_args, update=False):
        # Get the resource type to set the correct type
        rt_attrs = utils.get_client(self).resource_type.get(
            name=parsed_args.resource_type)['attributes']
        resource = {}
        if not update:
            resource['id'] = parsed_args.resource_id
        if parsed_args.attribute:
            for attr in parsed_args.attribute:
                attr, __, value = attr.partition(":")
                attr_type = rt_attrs.get(attr, {}).get('type')
                if attr_type == "number":
                    value = float(value)
                elif attr_type == "bool":
                    value = bool(distutils.util.strtobool(value))
                resource[attr] = value
        if (parsed_args.add_metric or
           parsed_args.create_metric or
           (update and parsed_args.delete_metric)):
            if update:
                r = utils.get_client(self).resource.get(
                    parsed_args.resource_type,
                    parsed_args.resource_id)
                default = r['metrics']
                for metric_name in parsed_args.delete_metric:
                    try:
                        del default[metric_name]
                    except KeyError:
                        raise exceptions.MetricNotFound(
                            message="Metric name %s not found" % metric_name)
            else:
                default = {}
            resource['metrics'] = default
            for metric in parsed_args.add_metric:
                name, _, value = metric.partition(":")
                resource['metrics'][name] = value
            for metric in parsed_args.create_metric:
                name, _, value = metric.partition(":")
                if value is "":
                    resource['metrics'][name] = {}
                else:
                    resource['metrics'][name] = {'archive_policy_name': value}

        return resource

    def take_action(self, parsed_args):
        resource = self._resource_from_args(parsed_args)
        res = utils.get_client(self).resource.create(
            resource_type=parsed_args.resource_type, resource=resource)
        if parsed_args.formatter == 'table':
            normalize_metrics(res)
        return self.dict2columns(res)


class CliResourceUpdate(CliResourceCreate):
    """Update a resource."""

    def get_parser(self, prog_name):
        parser = super(CliResourceUpdate, self).get_parser(prog_name)
        parser.add_argument("-d", "--delete-metric", action='append',
                            default=[],
                            help="Name of a metric to delete"),
        return parser

    def take_action(self, parsed_args):
        resource = self._resource_from_args(parsed_args, update=True)
        res = utils.get_client(self).resource.update(
            resource_type=parsed_args.resource_type,
            resource_id=parsed_args.resource_id,
            resource=resource)
        if parsed_args.formatter == 'table':
            normalize_metrics(res)
        return self.dict2columns(res)


class CliResourceDelete(command.Command):
    """Delete a resource."""

    def get_parser(self, prog_name):
        parser = super(CliResourceDelete, self).get_parser(prog_name)
        parser.add_argument("resource_id",
                            help="ID of the resource")
        return parser

    def take_action(self, parsed_args):
        utils.get_client(self).resource.delete(parsed_args.resource_id)


class CliResourceBatchDelete(show.ShowOne):
    """Delete a batch of resources based on attribute values."""

    def get_parser(self, prog_name):
        parser = super(CliResourceBatchDelete, self).get_parser(prog_name)
        parser.add_argument("--type", "-t", dest="resource_type",
                            default="generic", help="Type of resource")
        utils.add_query_argument("query", parser)
        return parser

    def take_action(self, parsed_args):
        res = utils.get_client(self).resource.batch_delete(
            resource_type=parsed_args.resource_type,
            query=parsed_args.query)
        return self.dict2columns(res)



# Copyright 2012-2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six


class Manager(object):
    DEFAULT_HEADERS = {
        "Accept": "application/json, */*",
    }

    def __init__(self, client):
        self.client = client

    def _set_default_headers(self, kwargs):
        headers = kwargs.get('headers', {})
        for k, v in six.iteritems(self.DEFAULT_HEADERS):
            if k not in headers:
                headers[k] = v
        kwargs['headers'] = headers
        return kwargs

    def _get(self, *args, **kwargs):
        self._set_default_headers(kwargs)
        return self.client.api.get(*args, **kwargs)

    def _post(self, *args, **kwargs):
        self._set_default_headers(kwargs)
        return self.client.api.post(*args, **kwargs)

    def _put(self, *args, **kwargs):
        self._set_default_headers(kwargs)
        return self.client.api.put(*args, **kwargs)

    def _patch(self, *args, **kwargs):
        self._set_default_headers(kwargs)
        return self.client.api.patch(*args, **kwargs)

    def _delete(self, *args, **kwargs):
        self._set_default_headers(kwargs)
        return self.client.api.delete(*args, **kwargs)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import uuid

from debtcollector import removals

import iso8601

import ujson

from gnocchiclient import utils
from gnocchiclient.v1 import base


class MetricManager(base.Manager):
    metric_url = "v1/metric/"
    resource_url = "v1/resource/generic/%s/metric/"
    metric_batch_url = "v1/batch/metrics/measures"
    resources_batch_url = "v1/batch/resources/metrics/measures"

    def list(self, limit=None, marker=None, sorts=None):
        """List metrics.

        :param limit: maximum number of resources to return
        :type limit: int
        :param marker: the last item of the previous page; we return the next
                       results after this value.
        :type marker: str
        :param sorts: list of resource attributes to order by. (example
                      ["user_id:desc-nullslast", "project_id:asc"]
        :type sorts: list of str
        """
        params = utils.build_pagination_options(False, False, limit, marker,
                                                sorts)
        metrics = []
        page_url = "%s?%s" % (self.metric_url[:-1],
                              utils.dict_to_querystring(params))
        while page_url:
            page = self._get(page_url)
            metrics.extend(page.json())
            if limit is None or len(metrics) < limit:
                page_url = page.links.get("next", {'url': None})['url']
            else:
                break
        return metrics

    @staticmethod
    def _ensure_metric_is_uuid(metric, attribute="resource_id"):
        try:
            uuid.UUID(metric)
        except ValueError:
            raise TypeError("%s is required to get a metric by name" %
                            attribute)

    def get(self, metric, resource_id=None):
        """Get an metric.

        :param metric: ID or Name of the metric
        :type metric: str
        :param resource_id: ID of the resource (required
                            to get a metric by name)
        :type resource_id: str
        """
        if resource_id is None:
            self._ensure_metric_is_uuid(metric)
            url = self.metric_url + metric
        else:
            url = (self.resource_url % resource_id) + metric
        return self._get(url).json()

    # FIXME(jd): This is what create will be after debtcollector warnings have
    # been removed. We provide it right now for the benchmark code, that can't
    # pickle a debtcollector-ed method.
    def _create_new(self, name=None, archive_policy_name=None,
                    resource_id=None, unit=None):
        """Create an metric.

        :param name: Metric name.
        :type name: str
        :param archive_policy_name: Archive policy name.
        :type archive_policy_name: str
        :param resource_id: The resource ID to attach the metric to.
        :type resource_id: str
        :param unit: The unit of the metric.
        :type unit: str
        """
        metric = {}
        if name is not None:
            metric["name"] = name
        if archive_policy_name is not None:
            metric["archive_policy_name"] = archive_policy_name
        if unit is not None:
            metric["unit"] = unit

        if resource_id is None:
            return self._post(
                self.metric_url, headers={'Content-Type': "application/json"},
                data=ujson.dumps(metric)).json()

        if name is None:
            raise TypeError(
                "Metric name is required if resource_id is set")

        return self._post(
            self.resource_url % resource_id,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps({name: metric})).json()[0]

    # FIXME(jd): remove refetch_metric when LP#1497171 is fixed
    @removals.removed_kwarg("refetch_metric")
    @removals.removed_kwarg("metric")
    def create(self, metric=None, refetch_metric=True,
               name=None,
               archive_policy_name=None,
               resource_id=None,
               unit=None):
        """Create an metric.

        :param name: Metric name.
        :type name: str
        :param archive_policy_name: Archive policy name.
        :type archive_policy_name: str
        :param resource_id: The resource ID to attach the metric to.
        :type resource_id: str
        :param unit: The unit of the metric.
        :type unit: str
        """
        if metric is None:
            metric = {}
            if name is not None:
                metric["name"] = name
            if archive_policy_name is not None:
                metric["archive_policy_name"] = archive_policy_name
            if resource_id is not None:
                metric["resource_id"] = resource_id
            if unit is not None:
                metric["unit"] = unit

        resource_id = metric.get('resource_id')

        if resource_id is None:
            metric = self._post(
                self.metric_url, headers={'Content-Type': "application/json"},
                data=ujson.dumps(metric)).json()
            # FIXME(sileht): create and get have a
            # different output: LP#1497171
            if refetch_metric:
                return self.get(metric["id"])
            return metric

        metric_name = metric.get('name')

        if metric_name is None:
            raise TypeError("metric_name is required if resource_id is set")

        del metric['resource_id']
        metric = {metric_name: metric}
        metric = self._post(
            self.resource_url % resource_id,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(metric))
        return self.get(metric_name, resource_id)

    def delete(self, metric, resource_id=None):
        """Delete an metric.

        :param metric: ID or Name of the metric
        :type metric: str
        :param resource_id: ID of the resource (required
                            to get a metric by name)
        :type resource_id: str
        """
        if resource_id is None:
            self._ensure_metric_is_uuid(metric)
            url = self.metric_url + metric
        else:
            url = self.resource_url % resource_id + metric
        self._delete(url)

    def add_measures(self, metric, measures, resource_id=None):
        """Add measurements to a metric.

        :param metric: ID or Name of the metric
        :type metric: str
        :param resource_id: ID of the resource (required
                            to get a metric by name)
        :type resource_id: str
        :param measures: measurements
        :type measures: list of dict(timestamp=timestamp, value=float)
        """
        if resource_id is None:
            self._ensure_metric_is_uuid(metric)
            url = self.metric_url + metric + "/measures"
        else:
            url = self.resource_url % resource_id + metric + "/measures"
        return self._post(
            url, headers={'Content-Type': "application/json"},
            data=ujson.dumps(measures))

    def batch_metrics_measures(self, measures):
        """Add measurements to metrics.

        :param measures: measurements
        :type dict(metric_id: list of dict(timestamp=timestamp, value=float))
        """
        return self._post(
            self.metric_batch_url,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(measures))

    def batch_resources_metrics_measures(self, measures, create_metrics=False):
        """Add measurements to named metrics if resources.

        :param measures: measurements
        :type dict(resource_id: dict(metric_name:
            list of dict(timestamp=timestamp, value=float)))
        """
        return self._post(
            self.resources_batch_url,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(measures),
            params=dict(create_metrics=create_metrics))

    def get_measures(self, metric, start=None, stop=None, aggregation=None,
                     granularity=None, resource_id=None, refresh=False,
                     resample=None, **kwargs):
        """Get measurements of a metric.

        :param metric: ID or Name of the metric
        :type metric: str
        :param start: beginning of the period
        :type start: timestamp
        :param stop: end of the period
        :type stop: timestamp
        :param aggregation: aggregation to retrieve
        :type aggregation: str
        :param granularity: granularity to retrieve (in seconds)
        :type granularity: int
        :param resource_id: ID of the resource (required
                            to get a metric by name)
        :type resource_id: str
        :param refresh: force aggregation of all known measures
        :type refresh: bool
        :param resample: resample measures to new granularity
        :type resample: float

        All other arguments are arguments are dedicated to custom aggregation
        method passed as-is to the Gnocchi.
        """
        if isinstance(start, datetime.datetime):
            start = start.isoformat()
        if isinstance(stop, datetime.datetime):
            stop = stop.isoformat()

        params = dict(start=start, stop=stop, aggregation=aggregation,
                      granularity=granularity, refresh=refresh,
                      resample=resample)
        params.update(kwargs)
        if resource_id is None:
            self._ensure_metric_is_uuid(metric)
            url = self.metric_url + metric + "/measures"
        else:
            url = self.resource_url % resource_id + metric + "/measures"
        measures = self._get(url, params=params).json()
        return [(iso8601.parse_date(ts), g, value)
                for ts, g, value in measures]

    def aggregation(self, metrics, query=None,
                    start=None, stop=None, aggregation=None,
                    reaggregation=None, granularity=None,
                    needed_overlap=None, resource_type="generic",
                    groupby=None, refresh=False, resample=None, fill=None):
        """Get measurements of an aggregated metrics.

        :param metrics: IDs of metric or metric name
        :type metric: list or str
        :param query: The query dictionary
        :type query: dict
        :param start: beginning of the period
        :type start: timestamp
        :param stop: end of the period
        :type stop: timestamp
        :param aggregation: granularity aggregation function to retrieve
        :type aggregation: str
        :param reaggregation: groupby aggregation function to retrieve
        :type reaggregation: str
        :param granularity: granularity to retrieve (in seconds)
        :type granularity: int
        :param needed_overlap: percent of datapoints in each metrics required
        :type needed_overlap: float
        :param resource_type: type of resource for the query
        :type resource_type: str
        :param groupby: list of attribute to group by
        :type groupby: list
        :param refresh: force aggregation of all known measures
        :type refresh: bool
        :param resample: resample measures to new granularity
        :type resample: float
        :param fill: value to use when backfilling missing datapoints
        :type fill: float or 'null'

        See Gnocchi REST API documentation for the format
        of *query dictionary*
        http://docs.openstack.org/developer/gnocchi/rest.html#searching-for-resources
        """
        if isinstance(start, datetime.datetime):
            start = start.isoformat()
        if isinstance(stop, datetime.datetime):
            stop = stop.isoformat()

        params = dict(start=start, stop=stop, aggregation=aggregation,
                      reaggregation=reaggregation, granularity=granularity,
                      needed_overlap=needed_overlap, groupby=groupby,
                      refresh=refresh, resample=resample, fill=fill)
        if query is None:
            for metric in metrics:
                self._ensure_metric_is_uuid(metric)
            params['metric'] = metrics
            measures = self._get("v1/aggregation/metric",
                                 params=params).json()
        else:
            if isinstance(query, dict):
                measures = self._post(
                    "v1/aggregation/resource/%s/metric/%s?%s" % (
                        resource_type, metrics,
                        utils.dict_to_querystring(params)),
                    headers={'Content-Type': "application/json"},
                    data=ujson.dumps(query)).json()
            else:
                params['filter'] = query
                measures = self._post(
                    "v1/aggregation/resource/%s/metric/%s?%s" % (
                        resource_type, metrics,
                        utils.dict_to_querystring(params)),
                    headers={'Content-Type': "application/json"}).json()
        if groupby is None:
            return [(iso8601.parse_date(ts), g, value)
                    for ts, g, value in measures]

        for group in measures:
            group["measures"] = [
                (iso8601.parse_date(ts), g, value)
                for ts, g, value in group["measures"]
            ]

        return measures



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import sys

from keystoneauth1 import adapter
from keystoneauth1 import exceptions as k_exc

from gnocchiclient import exceptions


def Client(version, *args, **kwargs):
    module = 'gnocchiclient.v%s.client' % version
    __import__(module)
    client_class = getattr(sys.modules[module], 'Client')
    return client_class(*args, **kwargs)


class SessionClient(adapter.Adapter):
    def request(self, url, method, **kwargs):
        kwargs.setdefault('headers', kwargs.get('headers', {}))
        # NOTE(sileht): The standard call raises errors from
        # keystoneauth, where we need to raise the gnocchiclient errors.
        raise_exc = kwargs.pop('raise_exc', True)

        try:
            resp = super(SessionClient, self).request(url,
                                                      method,
                                                      raise_exc=False,
                                                      **kwargs)
        except k_exc.connection.ConnectFailure as e:
            raise exceptions.ConnectionFailure(
                message=str(e), url=url, method=method)
        except k_exc.connection.UnknownConnectionError as e:
            raise exceptions.UnknownConnectionError(
                message=str(e), url=url, method=method)
        except k_exc.connection.ConnectTimeout as e:
            raise exceptions.ConnectionTimeout(
                message=str(e), url=url, method=method)
        except k_exc.SSLError as e:
            raise exceptions.SSLError(message=str(e), url=url, method=method)

        if raise_exc and resp.status_code >= 400:
            raise exceptions.from_response(resp, method)
        return resp



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import uuid

from gnocchiclient.tests.functional import base


class ArchivePolicyClientTest(base.ClientTestBase):
    def test_archive_policy_scenario(self):
        # CREATE
        apname = str(uuid.uuid4())
        result = self.gnocchi(
            u'archive-policy', params=u"create %s"
            u" --back-window 0"
            u" -d granularity:1s,points:86400" % apname)
        policy = json.loads(result)
        self.assertEqual(apname, policy["name"])

        # CREATE FAIL
        result = self.gnocchi(
            u'archive-policy', params=u"create %s"
            u" --back-window 0"
            u" -d granularity:1s,points:86400" % apname,
            fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Archive policy %s already exists (HTTP 409)\n" % apname,
            result)

        # GET
        result = self.gnocchi(
            'archive-policy', params="show %s" % apname)
        policy = json.loads(result)
        self.assertEqual(apname, policy["name"])

        # LIST
        result = self.gnocchi(
            'archive-policy', params="list")
        policies = json.loads(result)
        policy_from_list = [p for p in policies
                            if p['name'] == apname][0]
        for field in ["back_window", "definition", "aggregation_methods"]:
            self.assertEqual(policy[field], policy_from_list[field])

        # UPDATE
        result = self.gnocchi(
            'archive-policy', params='update %s'
            ' -d granularity:1s,points:60' % apname)
        policy = json.loads(result)
        self.assertEqual(apname, policy["name"])

        # UPDATE FAIL
        result = self.gnocchi(
            'archive-policy', params='update %s'
            ' -d granularity:5s,points:86400' % apname,
            fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Archive policy %s does not support change: 1.0 granularity "
            "interval was changed (HTTP 400)\n" % apname,
            result)

        # DELETE
        self.gnocchi('archive-policy',
                     params="delete %s" % apname,
                     has_output=False)

        # GET FAIL
        result = self.gnocchi('archive-policy',
                              params="show %s" % apname,
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Archive policy %s does not exist (HTTP 404)\n" % apname,
            result)

        # DELETE FAIL
        result = self.gnocchi('archive-policy',
                              params="delete %s" % apname,
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Archive policy %s does not exist (HTTP 404)\n" % apname,
            result)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import distutils.util

from cliff import command
from cliff import lister
from cliff import show

from gnocchiclient import utils


class CliResourceTypeList(lister.Lister):
    """List resource types."""

    COLS = ('name', 'attributes')

    def take_action(self, parsed_args):
        resource_types = utils.get_client(self).resource_type.list()
        for resource_type in resource_types:
            resource_type['attributes'] = utils.format_dict_dict(
                resource_type['attributes'])
        return utils.list2cols(self.COLS, resource_types)


class CliResourceTypeCreate(show.ShowOne):
    """Create a resource type."""

    def get_parser(self, prog_name):
        parser = super(CliResourceTypeCreate, self).get_parser(prog_name)
        parser.add_argument("name", help="name of the resource type")
        parser.add_argument("-a", "--attribute", action='append',
                            type=self._resource_attribute,
                            default=[],
                            help=(u"attribute definition, "
                                  u"attribute_name:"
                                  u"attribute_type:"
                                  u"attribute_is_required:"
                                  u"attribute_type_option_name="
                                  u"attribute_type_option_value:... "
                                  u"For example: "
                                  u"display_name:string:true:max_length=255"))
        return parser

    @classmethod
    def _resource_attribute(cls, value):
        config = value.split(":")
        name = config.pop(0)
        attrs = {}
        if config:
            attrs["type"] = config.pop(0)
        if config:
            attrs["required"] = bool(distutils.util.strtobool(config.pop(0)))
        while config:
            param, _, value = config.pop(0).partition("=")
            opts = attrs
            if param == 'fill':
                opts = attrs.setdefault("options", {})
            try:
                opts[param] = int(value)
            except ValueError:
                try:
                    opts[param] = float(value)
                except ValueError:
                    opts[param] = value
        return (name, attrs)

    def take_action(self, parsed_args):
        resource_type = {'name': parsed_args.name}
        if parsed_args.attribute:
            resource_type['attributes'] = dict(parsed_args.attribute)
        res = utils.get_client(self).resource_type.create(
            resource_type=resource_type)
        utils.format_resource_type(res)
        return self.dict2columns(res)


class CliResourceTypeUpdate(CliResourceTypeCreate):
    def get_parser(self, prog_name):
        parser = super(CliResourceTypeUpdate, self).get_parser(prog_name)
        parser.add_argument("-r", "--remove-attribute", action='append',
                            default=[],
                            help=u"attribute name")

        return parser

    def take_action(self, parsed_args):
        operations = []
        if parsed_args.attribute:
            for name, attrs in parsed_args.attribute:
                operations.append({'op': 'add',
                                   'path': '/attributes/%s' % name,
                                   'value': attrs})
        if parsed_args.remove_attribute:
            for name in parsed_args.remove_attribute:
                operations.append({'op': 'remove',
                                   'path': '/attributes/%s' % name})

        res = utils.get_client(self).resource_type.update(
            parsed_args.name, operations)
        utils.format_resource_type(res)
        return self.dict2columns(res)


class CliResourceTypeShow(show.ShowOne):
    """Show a resource type."""

    def get_parser(self, prog_name):
        parser = super(CliResourceTypeShow, self).get_parser(prog_name)
        parser.add_argument("name", help="name of the resource type")
        return parser

    def take_action(self, parsed_args):
        res = utils.get_client(self).resource_type.get(name=parsed_args.name)
        utils.format_resource_type(res)
        return self.dict2columns(res)


class CliResourceTypeDelete(command.Command):
    """Delete a resource type."""

    def get_parser(self, prog_name):
        parser = super(CliResourceTypeDelete, self).get_parser(prog_name)
        parser.add_argument("name", help="name of the resource type")
        return parser

    def take_action(self, parsed_args):
        utils.get_client(self).resource_type.delete(parsed_args.name)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cliff import command
from cliff import lister
from cliff import show

from gnocchiclient import utils


class CliArchivePolicyRuleList(lister.Lister):
    """List archive policy rules."""

    COLS = ('name', 'archive_policy_name', 'metric_pattern')

    def take_action(self, parsed_args):
        ap_rules = utils.get_client(self).archive_policy_rule.list()
        return utils.list2cols(self.COLS, ap_rules)


class CliArchivePolicyRuleShow(show.ShowOne):
    """Show an archive policy rule."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyRuleShow, self).get_parser(prog_name)
        parser.add_argument("name",
                            help="Name of the archive policy rule")
        return parser

    def take_action(self, parsed_args):
        ap_rule = utils.get_client(self).archive_policy_rule.get(
            name=parsed_args.name)
        return self.dict2columns(ap_rule)


class CliArchivePolicyRuleCreate(show.ShowOne):
    """Create an archive policy rule."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyRuleCreate, self).get_parser(prog_name)
        parser.add_argument("name",
                            help="Rule name")
        parser.add_argument("-a", "--archive-policy-name",
                            dest="archive_policy_name",
                            required=True,
                            help="Archive policy name")
        parser.add_argument("-m", "--metric-pattern",
                            dest="metric_pattern", required=True,
                            help="Wildcard of metric name to match")
        return parser

    def take_action(self, parsed_args):
        rule = utils.dict_from_parsed_args(
            parsed_args, ["name", "metric_pattern", "archive_policy_name"])
        policy = utils.get_client(self).archive_policy_rule.create(rule)
        return self.dict2columns(policy)


class CliArchivePolicyRuleUpdate(show.ShowOne):
    """Update an archive policy rule."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyRuleUpdate, self).get_parser(prog_name)
        parser.add_argument("name",
                            help="Rule name")
        parser.add_argument("-n", "--name",
                            dest="new_name",
                            required=True,
                            help="New rule name")
        return parser

    def take_action(self, parsed_args):
        policy = utils.get_client(self).archive_policy_rule.update(
            parsed_args.name,
            parsed_args.new_name)
        return self.dict2columns(policy)


class CliArchivePolicyRuleDelete(command.Command):
    """Delete an archive policy rule."""

    def get_parser(self, prog_name):
        parser = super(CliArchivePolicyRuleDelete, self).get_parser(prog_name)
        parser.add_argument("name",
                            help="Name of the archive policy rule")
        return parser

    def take_action(self, parsed_args):
        utils.get_client(self).archive_policy_rule.delete(parsed_args.name)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import keystoneauth1.session

from gnocchiclient import client
from gnocchiclient.v1 import aggregates
from gnocchiclient.v1 import archive_policy
from gnocchiclient.v1 import archive_policy_rule
from gnocchiclient.v1 import build
from gnocchiclient.v1 import capabilities
from gnocchiclient.v1 import metric
from gnocchiclient.v1 import resource
from gnocchiclient.v1 import resource_type
from gnocchiclient.v1 import status


class Client(object):
    """Client for the Gnocchi v1 API.

    :param session: keystoneauth1 session
    :type session: py:class:`keystoneauth1.session.Session` (optional)
    :param adapter_options: options to pass to
                            py:class:`keystoneauth1.adapter.Adapter`
    :type adapter_options: dict (optional)
    :param session_options: options to pass to
                            py:class:`keystoneauth1.session.Session`
    :type session_options: dict (optional)
    """

    def __init__(self, session=None, adapter_options=None,
                 session_options=None):
        """Initialize a new client for the Gnocchi v1 API."""
        session_options = session_options or {}
        adapter_options = adapter_options or {}

        adapter_options.setdefault('service_type', "metric")

        if session is None:
            session = keystoneauth1.session.Session(**session_options)
        else:
            if session_options:
                raise ValueError("session and session_options are exclusive")

        self.api = client.SessionClient(session, **adapter_options)
        self.resource = resource.ResourceManager(self)
        self.resource_type = resource_type.ResourceTypeManager(self)
        self.archive_policy = archive_policy.ArchivePolicyManager(self)
        self.archive_policy_rule = (
            archive_policy_rule.ArchivePolicyRuleManager(self))
        self.metric = metric.MetricManager(self)
        self.aggregates = aggregates.AggregatesManager(self)
        self.capabilities = capabilities.CapabilitiesManager(self)
        self.status = status.StatusManager(self)
        self.build = build.BuildManager(self)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import uuid

from gnocchiclient.tests.functional import base


class BenchmarkMetricTest(base.ClientTestBase):
    def test_benchmark_metric_create_wrong_workers(self):
        result = self.gnocchi(
            u'benchmark', params=u"metric create -n 0",
            fail_ok=True, merge_stderr=True)
        self.assertIn("0 must be greater than 0", result)

    def test_benchmark_metric_create(self):
        apname = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create %s "
                     "--back-window 0 -d granularity:1s,points:86400" % apname)

        result = self.gnocchi(
            u'benchmark', params=u"metric create -n 10 -a %s" % apname)
        result = json.loads(result)
        self.assertEqual(10, int(result['create executed']))
        self.assertLessEqual(int(result['create failures']), 10)
        self.assertLessEqual(int(result['delete executed']),
                             int(result['create executed']))

        result = self.gnocchi(
            u'benchmark', params=u"metric create -k -n 10 -a %s" % apname)
        result = json.loads(result)
        self.assertEqual(10, int(result['create executed']))
        self.assertLessEqual(int(result['create failures']), 10)
        self.assertNotIn('delete executed', result)

    def test_benchmark_metric_get(self):
        apname = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create %s "
                     "--back-window 0 -d granularity:1s,points:86400" % apname)

        result = self.gnocchi(
            u'metric', params=u"create -a %s" % apname)
        metric = json.loads(result)

        result = self.gnocchi(
            u'benchmark', params=u"metric show -n 10 %s" % metric['id'])
        result = json.loads(result)
        self.assertEqual(10, int(result['show executed']))
        self.assertLessEqual(int(result['show failures']), 10)

    def test_benchmark_measures_add(self):
        apname = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create %s "
                     "--back-window 0 -d granularity:1s,points:86400" % apname)

        result = self.gnocchi(
            u'metric', params=u"create -a %s" % apname)
        metric = json.loads(result)

        result = self.gnocchi(
            u'benchmark', params=u"measures add -n 10 -b 4 %s" % metric['id'])
        result = json.loads(result)
        self.assertEqual(2, int(result['push executed']))
        self.assertLessEqual(int(result['push failures']), 2)

        result = self.gnocchi(
            u'benchmark',
            params=u"measures add -s 2010-01-01 -n 10 -b 4 %s"
            % metric['id'])
        result = json.loads(result)
        self.assertEqual(2, int(result['push executed']))
        self.assertLessEqual(int(result['push failures']), 2)

        result = self.gnocchi(
            u'benchmark',
            params=u"measures add --wait -s 2010-01-01 -n 10 -b 4 %s"
            % metric['id'])
        result = json.loads(result)
        self.assertEqual(2, int(result['push executed']))
        self.assertLessEqual(int(result['push failures']), 2)
        self.assertIn("extra wait to process measures", result)

    def test_benchmark_measures_show(self):
        apname = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create %s "
                     "--back-window 0 -d granularity:1s,points:86400" % apname)

        result = self.gnocchi(
            u'metric', params=u"create -a %s" % apname)
        metric = json.loads(result)

        result = self.gnocchi(
            u'benchmark',
            params=u"measures show -n 2 %s"
            % metric['id'])
        result = json.loads(result)
        self.assertEqual(2, int(result['show executed']))
        self.assertLessEqual(int(result['show failures']), 2)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cliff import lister

from gnocchiclient import utils


class CliAggregates(lister.Lister):
    """Get measurements of aggregated metrics."""

    COLS = ('name', 'timestamp', 'granularity', 'value')

    def get_parser(self, prog_name):
        parser = super(CliAggregates, self).get_parser(prog_name)
        parser.add_argument("operations",
                            help="Operations to apply to time series")
        utils.add_query_argument("search", parser, nargs="?", default=None)
        parser.add_argument("--resource-type", default="generic",
                            help="Resource type to query"),
        parser.add_argument("--start",
                            help="beginning of the period")
        parser.add_argument("--stop",
                            help="end of the period")
        parser.add_argument("--granularity",
                            help="granularity to retrieve")
        parser.add_argument("--needed-overlap", type=float,
                            help=("percentage of overlap across datapoints"))
        parser.add_argument("--groupby",
                            action='append',
                            help="Attribute to use to group resources"),
        parser.add_argument("--fill",
                            help=("Value to use when backfilling timestamps "
                                  "with missing values in a subset of series. "
                                  "Value should be a float or 'null'."))
        return parser

    def take_action(self, parsed_args):
        aggregates = utils.get_client(self).aggregates.fetch(
            operations=parsed_args.operations,
            resource_type=parsed_args.resource_type,
            search=parsed_args.search,
            start=parsed_args.start,
            stop=parsed_args.stop,
            granularity=parsed_args.granularity,
            needed_overlap=parsed_args.needed_overlap,
            groupby=parsed_args.groupby,
        )

        if parsed_args.search and parsed_args.groupby:
            ms = []
            for g in aggregates:
                group_name = ", ".join("%s: %s" % (k, g['group'][k])
                                       for k in sorted(g['group']))
                for row in self.flatten_measures(g["measures"]["measures"]):
                    ms.append((group_name, ) + row)
            return ('group',) + self.COLS, ms
        return self.COLS, list(self.flatten_measures(aggregates["measures"]))

    @classmethod
    def flatten_measures(cls, data, labels=None):
        if labels is None:
            labels = tuple()
        for key in data:
            if isinstance(data[key], list):
                name = "/".join(labels + (key, ))
                for ts, g, value in data[key]:
                    yield (name, ts.isoformat(), g, value)
            else:
                for row in cls.flatten_measures(data[key], labels + (key,)):
                    yield row



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import os
import tempfile
import uuid

from gnocchiclient import auth
from gnocchiclient import client
from gnocchiclient import utils
from gnocchiclient.tests.functional import base


class MetricClientTest(base.ClientTestBase):
    def test_delete_several_metrics(self):
        apname = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create %s "
                     "--back-window 0 -d granularity:1s,points:86400" % apname)
        # Create 2 metrics
        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name %s" % apname)
        metric1 = json.loads(result)

        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name %s" % apname)
        metric2 = json.loads(result)

        # DELETE
        self.gnocchi('metric', params="delete %s %s"
                     % (metric1["id"], metric2["id"]),
                     has_output=False)

        # GET FAIL
        result = self.gnocchi('metric', params="show %s" % metric1["id"],
                              fail_ok=True, merge_stderr=True)
        self.assertEqual("Metric %s does not exist (HTTP 404)\n" %
                         metric1["id"], result)
        result = self.gnocchi('metric', params="show %s" % metric2["id"],
                              fail_ok=True, merge_stderr=True)
        self.assertEqual("Metric %s does not exist (HTTP 404)\n" %
                         metric2["id"], result)

    def test_metric_listing(self):
        apname = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create "
                     "-d granularity:1s,points:86400 %s" % apname)
        # Create 1005 metrics
        c = client.Client(
            1,
            session_options={
                'auth': auth.GnocchiBasicPlugin(
                    user="admin", endpoint=self.endpoint)
            },
        )
        for i in range(1005):
            c.metric.create({"archive_policy_name": apname})

        result = self.gnocchi(u'metric', params=u"list")
        self.assertGreaterEqual(len(json.loads(result)), 1005)

        result = self.gnocchi(u'metric', params=u"list"
                              u" --limit 2")
        self.assertEqual(2, len(json.loads(result)))

    def test_metric_measures_exhautic_format(self):
        ap_name = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create " + ap_name +
                     " --back-window 0 -d granularity:1s,points:86400")
        # CREATE METRIC
        metric_name = str(uuid.uuid4())
        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name " + ap_name + " " + metric_name)
        metric = json.loads(result)

        # MEASURES ADD
        self.gnocchi('measures',
                     params=("add %s "
                             "--measure 'now@42' ")
                     % metric["id"], has_output=False)

        # MEASURES SHOW
        result = self.gnocchi('measures', params="show --refresh " +
                              metric["id"])
        measures = json.loads(result)
        self.assertEqual(1, len(measures))
        self.assertEqual(42, measures[0]['value'])

    def test_metric_measures_show_tz(self):
        ap_name = str(uuid.uuid4())
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create " + ap_name +
                     " --back-window 0 -d granularity:1s,points:86400")
        # CREATE METRIC
        metric_name = str(uuid.uuid4())
        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name " + ap_name + " " + metric_name)
        metric = json.loads(result)

        # MEASURES ADD
        self.gnocchi('measures',
                     params=("add %s "
                             "-m '2015-03-06T14:33:57Z@43.11' "
                             "--measure '2015-03-06T16:34:12+02:00@12' ")
                     % metric["id"], has_output=False)

        # MEASURES SHOW
        def _delete_tz():
            os.unsetenv("TZ")
        self.addCleanup(_delete_tz)
        os.putenv("TZ", "Europe/Paris")
        result = self.gnocchi('measures', params="show --refresh " +
                              metric["id"])
        measures = json.loads(result)
        self.assertEqual("2015-03-06T15:33:57+01:00", measures[0]['timestamp'])
        self.assertEqual("2015-03-06T15:34:12+01:00", measures[1]['timestamp'])

        # MEASURES SHOW
        result = self.gnocchi('measures', params="show --utc --refresh " +
                              metric["id"])
        measures = json.loads(result)
        # Check that --utc is honored
        self.assertEqual("2015-03-06T14:33:57+00:00", measures[0]['timestamp'])
        self.assertEqual("2015-03-06T14:34:12+00:00", measures[1]['timestamp'])

    def test_metric_scenario(self):
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create metric-test "
                     "--back-window 0 -d granularity:1s,points:86400")

        # CREATE WITH NAME AND WITHOUT UNIT
        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name metric-test some-name")
        metric = json.loads(result)
        self.assertIsNotNone(metric["id"])
        self.assertEqual("admin", metric["creator"])
        self.assertEqual('some-name', metric["name"])
        self.assertIsNone(metric["unit"])
        self.assertIsNone(metric["resource_id"])
        self.assertIn("metric-test", metric["archive_policy_name"])

        # CREATE WITH UNIT
        result = self.gnocchi(
            u'metric', params=u"create another-name"
            u" --archive-policy-name metric-test"
            u" --unit some-unit")
        metric = json.loads(result)
        self.assertIsNotNone(metric["id"])
        self.assertEqual("admin", metric["creator"])
        self.assertEqual('another-name', metric["name"])
        self.assertEqual('some-unit', metric["unit"])
        self.assertIsNone(metric["resource_id"])
        self.assertIn("metric-test", metric["archive_policy_name"])

        # GET
        result = self.gnocchi('metric', params="show %s" % metric["id"])
        metric_get = json.loads(result)
        # `metric show` format resource/*, `metric create` does not
        metric_get['resource_id'] = metric_get['resource/id']
        del metric_get['resource/id']
        metric_get['archive_policy_name'] = metric_get['archive_policy/name']
        for k in list(metric_get):
            if k.startswith("archive_policy/"):
                del metric_get[k]
        self.assertEqual(metric, metric_get)

        # MEASURES ADD
        self.gnocchi('measures',
                     params=("add %s "
                             "-m '2015-03-06T14:33:57Z@43.11' "
                             "--measure '2015-03-06T14:34:12Z@12' ")
                     % metric["id"], has_output=False)

        # MEASURES GET with refresh
        result = self.gnocchi('measures',
                              params=("show %s "
                                      "--aggregation mean "
                                      "--granularity 1 "
                                      "--start 2015-03-06T14:32:00Z "
                                      "--stop 2015-03-06T14:36:00Z "
                                      "--refresh") % metric["id"])
        measures = json.loads(result)
        self.assertEqual(
            [{'granularity': 1.0,
              'timestamp': utils.dt_to_localtz(
                  utils.parse_date('2015-03-06T14:33:57+00:00')).isoformat(),
              'value': 43.11},
             {'granularity': 1.0,
              'timestamp': utils.dt_to_localtz(
                  utils.parse_date('2015-03-06T14:34:12+00:00')).isoformat(),
              'value': 12.0}], measures)

        # MEASURES GET
        result = self.retry_gnocchi(
            5, 'measures', params=("show %s "
                                   "--aggregation mean "
                                   "--granularity 1 "
                                   "--start 2015-03-06T14:32:00Z "
                                   "--stop 2015-03-06T14:36:00Z"
                                   ) % metric["id"])
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0}
        ], measures)

        # MEASURES GET RESAMPLE
        result = self.retry_gnocchi(
            5, 'measures', params=("show %s "
                                   "--aggregation mean "
                                   "--granularity 1 --resample 3600 "
                                   "--start 2015-03-06T14:32:00Z "
                                   "--stop 2015-03-06T14:36:00Z"
                                   ) % metric["id"])
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 3600.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:00:00+00:00')).isoformat(),
             'value': 27.555}
        ], measures)

        # MEASURES AGGREGATION
        result = self.gnocchi(
            'measures', params=("aggregation "
                                "--metric %s "
                                "--aggregation mean "
                                "--reaggregation sum "
                                "--granularity 1 "
                                "--start 2015-03-06T14:32:00Z "
                                "--stop 2015-03-06T14:36:00Z"
                                ) % metric["id"])
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0}
        ], measures)

        # BATCHING
        measures = json.dumps({
            metric['id']: [{'timestamp': '2015-03-06T14:34:12',
                            'value': 12}]})
        tmpfile = tempfile.NamedTemporaryFile(delete=False)
        self.addCleanup(os.remove, tmpfile.name)
        with tmpfile as f:
            f.write(measures.encode('utf8'))
        self.gnocchi('measures', params=("batch-metrics %s" % tmpfile.name),
                     has_output=False)
        self.gnocchi('measures', params="batch-metrics -",
                     input=measures.encode('utf8'),
                     has_output=False)

        # LIST
        result = self.gnocchi('metric', params="list")
        metrics = json.loads(result)
        metric_from_list = [p for p in metrics
                            if p['id'] == metric['id']][0]
        metric['archive_policy/name'] = metric['archive_policy_name']
        for field in ("id", "archive_policy/name", "name",
                      "unit", "resource_id"):
            self.assertEqual(metric[field], metric_from_list[field], field)

        # LIST + limit
        result = self.gnocchi('metric',
                              params=("list "
                                      "--sort name:asc "
                                      "--marker %s "
                                      "--limit 1") % metric['id'])
        metrics = json.loads(result)
        metric_from_list = metrics[0]
        self.assertEqual(1, len(metrics))
        self.assertTrue(metric['name'] < metric_from_list['name'])

        # DELETE
        self.gnocchi('metric', params="delete %s" % metric["id"],
                     has_output=False)

        # GET FAIL
        result = self.gnocchi('metric', params="show %s" % metric["id"],
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Metric %s does not exist (HTTP 404)\n" % metric["id"],
            result)

        # DELETE FAIL
        result = self.gnocchi('metric', params="delete %s" % metric["id"],
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Metric %s does not exist (HTTP 404)\n" % metric["id"],
            result)

    def test_metric_by_name_scenario(self):
        # PREPARE REQUIREMENT
        self.gnocchi("archive-policy", params="create metric-test2 "
                     "--back-window 0 -d granularity:1s,points:86400")
        self.gnocchi("resource", params="create metric-res")

        metric_name = str(uuid.uuid4())
        # CREATE
        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name metric-test2 -r metric-res %s"
            u" --unit some-unit" % metric_name)
        metric = json.loads(result)
        self.assertIsNotNone(metric["id"])
        self.assertEqual("admin", metric['creator'])
        self.assertEqual(metric_name, metric["name"])
        self.assertEqual('some-unit', metric["unit"])
        self.assertIsNotNone(metric["resource_id"])
        self.assertIn("metric-test", metric["archive_policy/name"])

        # CREATE FAIL
        result = self.gnocchi(
            u'metric', params=u"create"
            u" --archive-policy-name metric-test2 "
            "-r metric-res %s" % metric_name,
            fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Named metric %s already exists (HTTP 409)\n" % metric_name,
            result)

        # GET
        result = self.gnocchi('metric',
                              params="show -r metric-res %s" % metric_name)
        metric_get = json.loads(result)
        # `metric show` returns resource details, `metric create` does not
        metric_get['resource_id'] = metric_get['resource/id']
        for k in list(metric_get):
            if k.startswith("resource/"):
                del metric_get[k]
        self.assertEqual(metric, metric_get)

        # MEASURES ADD
        self.gnocchi('measures',
                     params=("add " + metric_name + " -r metric-res "
                             "-m '2015-03-06T14:33:57Z@43.11' "
                             "--measure '2015-03-06T14:34:12Z@12'"),
                     has_output=False)

        # MEASURES AGGREGATION with refresh
        result = self.gnocchi(
            'measures', params=("aggregation "
                                "--query \"id='metric-res'\" "
                                "--resource-type \"generic\" "
                                "-m " + metric_name + " "
                                "--aggregation mean "
                                "--needed-overlap 0 "
                                "--start 2015-03-06T14:32:00Z "
                                "--stop 2015-03-06T14:36:00Z "
                                "--refresh"))
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0}
        ], measures)

        # MEASURES AGGREGATION
        result = self.gnocchi(
            'measures', params=("aggregation "
                                "--query \"id='metric-res'\" "
                                "--resource-type \"generic\" "
                                "-m  " + metric_name + " "
                                "--aggregation mean "
                                "--needed-overlap 0 "
                                "--start 2015-03-06T14:32:00Z "
                                "--stop 2015-03-06T14:36:00Z"))
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0}
        ], measures)

        # MEASURES AGGREGATION WITH FILL
        result = self.gnocchi(
            'measures', params=("aggregation "
                                "--query \"id='metric-res'\" "
                                "--resource-type \"generic\" "
                                "-m  " + metric_name + " "
                                "--fill 0 "
                                "--granularity 1 "
                                "--start 2015-03-06T14:32:00Z "
                                "--stop 2015-03-06T14:36:00Z"))
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0}
        ], measures)

        # MEASURES AGGREGATION RESAMPLE
        result = self.gnocchi(
            'measures', params=("aggregation "
                                "--query \"id='metric-res'\" "
                                "--resource-type \"generic\" "
                                "-m  " + metric_name + " "
                                "--granularity 1 "
                                "--aggregation mean --resample=3600 "
                                "--needed-overlap 0 "
                                "--start 2015-03-06T14:32:00Z "
                                "--stop 2015-03-06T14:36:00Z"))
        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 3600.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date('2015-03-06T14:00:00+00:00')).isoformat(),
             'value': 27.555}
        ], measures)

        # MEASURES AGGREGATION GROUPBY
        result = self.gnocchi(
            'measures', params=("aggregation "
                                "--groupby project_id "
                                "--groupby user_id "
                                "--query \"id='metric-res'\" "
                                "--resource-type \"generic\" "
                                "-m  " + metric_name + " "
                                "--aggregation mean "
                                "--needed-overlap 0 "
                                "--start 2015-03-06T14:32:00Z "
                                "--stop 2015-03-06T14:36:00Z"))
        measures = json.loads(result)
        self.assertEqual([
            {'group': 'project_id: None, user_id: None',
             'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'group': 'project_id: None, user_id: None',
             'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0},
        ], measures)

        # MEASURES GET
        result = self.gnocchi('measures',
                              params=("show " + metric_name + " -r metric-res "
                                      "--aggregation mean "
                                      "--start 2015-03-06T14:32:00Z "
                                      "--stop 2015-03-06T14:36:00Z"))

        measures = json.loads(result)
        self.assertEqual([
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:33:57+00:00')).isoformat(),
             'value': 43.11},
            {'granularity': 1.0,
             'timestamp': utils.dt_to_localtz(
                 utils.parse_date(u'2015-03-06T14:34:12+00:00')).isoformat(),
             'value': 12.0},
        ], measures)

        # BATCHING
        measures = json.dumps({'metric-res': {metric_name: [{
            'timestamp': '2015-03-06T14:34:12', 'value': 12
        }]}})
        tmpfile = tempfile.NamedTemporaryFile(delete=False)
        self.addCleanup(os.remove, tmpfile.name)
        with tmpfile as f:
            f.write(measures.encode('utf8'))

        self.gnocchi('measures', params=("batch-resources-metrics %s" %
                                         tmpfile.name),
                     has_output=False)
        self.gnocchi('measures', params="batch-resources-metrics -",
                     input=measures.encode('utf8'),
                     has_output=False)

        # BATCHING --create-metrics
        measures = json.dumps({'metric-res': {'unknown-metric-name': [{
            'timestamp': '2015-03-06T14:34:12', 'value': 12
        }]}})
        self.gnocchi('measures',
                     params="batch-resources-metrics --create-metrics -",
                     input=measures.encode('utf8'), has_output=False)

        # LIST
        result = self.gnocchi('metric', params="list")
        metrics = json.loads(result)
        metric_from_list = [p for p in metrics
                            if p['archive_policy/name'] == 'metric-test2'][0]
        for field in ["archive_policy/name", "name"]:
            # FIXME(sileht): add "resource_id" or "resource"
            # when LP#1497171 is fixed
            self.assertEqual(metric[field], metric_from_list[field])

        # DELETE
        self.gnocchi('metric',
                     params="delete -r metric-res " + metric_name,
                     has_output=False)

        # GET FAIL
        result = self.gnocchi('metric',
                              params="show -r metric-res " + metric_name,
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Metric " + metric_name + " does not exist (HTTP 404)\n", result)

        # DELETE FAIL
        result = self.gnocchi('metric',
                              params="delete -r metric-res " + metric_name,
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Metric " + metric_name + " does not exist (HTTP 404)\n",
            result)

        # GET RESOURCE ID
        result = self.gnocchi(
            'resource', params="show -t generic metric-res")
        resource_id = json.loads(result)["id"]

        # DELETE RESOURCE
        self.gnocchi('resource', params="delete metric-res",
                     has_output=False)

        # GET FAIL WITH RESOURCE ERROR
        result = self.gnocchi('metric',
                              params="show " + metric_name + " -r metric-res",
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Resource %s does not exist (HTTP 404)\n" % resource_id,
            result)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ujson

from gnocchiclient.v1 import base


class ResourceTypeManager(base.Manager):
    url = "v1/resource_type/"

    def list(self):
        """List resource types."""
        return self._get(self.url).json()

    def create(self, resource_type):
        """Create a resource type.

        :param resource_type: resource type
        :type resource_type: dict
        """
        return self._post(
            self.url,
            headers={'Content-Type': "application/json"},
            data=ujson.dumps(resource_type)).json()

    def get(self, name):
        """Get a resource type.

        :param name: name of the resource type
        :type name: str
        """
        return self._get(self.url + name,
                         headers={'Content-Type': "application/json"}).json()

    def delete(self, name):
        """Delete a resource type.

        :param resource_type: Resource type
        :type resource_type: dict
        """
        self._delete(self.url + name)

    def update(self, name, operations):
        """Update a resource type.

        :param name: name of the resource type
        :type name: str
        :param operations: operations in RFC6902 format
        :type name: list
        """
        return self._patch(
            self.url + name,
            headers={'Content-Type': "application/json-patch+json"},
            data=ujson.dumps(operations)).json()



# -*- encoding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import absolute_import

import fnmatch

from cliff import sphinxext

from gnocchiclient import shell


class GnocchiAutoDocDirective(sphinxext.AutoprogramCliffDirective):
    def _load_commands(self):
        command_pattern = self.options.get('command')
        full_cmd_list = shell.GnocchiCommandManager.SHELL_COMMANDS.keys()
        if command_pattern:
            commands = [x for x in full_cmd_list
                        if fnmatch.fnmatch(x, command_pattern)]
        else:
            commands = full_cmd_list
        return dict((name, shell.GnocchiCommandManager.SHELL_COMMANDS[name])
                    for name in commands)


def setup(app):
    app.add_directive('autodoc-gnocchi', GnocchiAutoDocDirective)
    app.add_config_value('autoprogram_cliff_application', 'gnocchi', True)
    app.add_config_value('autoprogram_cliff_ignored', ['--help'], True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json

from gnocchiclient.tests.functional import base


class AggregatesClientTest(base.ClientTestBase):
    def test_scenario(self):
        # PREPARE AN ARCHIVE POLICY
        self.gnocchi("archive-policy", params="create agg-fetch-test "
                     "--back-window 0 -d granularity:1s,points:86400")

        r1 = json.loads(self.gnocchi("resource", params="create metric-res1"))
        r2 = json.loads(self.gnocchi("resource", params="create metric-res2"))

        # CREATE A METRIC
        result = self.gnocchi(
            u'metric', params=u"create"
            u' -r metric-res1'
            u" --archive-policy-name agg-fetch-test metric-name")
        metric1 = json.loads(result)
        self.assertIsNotNone(metric1["id"])
        self.assertEqual("admin", metric1["creator"])
        self.assertEqual('metric-name', metric1["name"])
        self.assertIsNone(metric1["unit"])
        self.assertIsNotNone(metric1["resource_id"])
        self.assertIn("agg-fetch-test", metric1["archive_policy/name"])

        # CREATE ANOTHER METRIC
        result = self.gnocchi(
            u'metric', params=u"create"
            u' -r metric-res2'
            u" --archive-policy-name agg-fetch-test"
            u" --unit some-unit metric-name")
        metric2 = json.loads(result)
        self.assertIsNotNone(metric2["id"])
        self.assertEqual("admin", metric2["creator"])
        self.assertEqual('metric-name', metric2["name"])
        self.assertEqual('some-unit', metric2["unit"])
        self.assertIsNotNone(metric2["resource_id"])
        self.assertIn("agg-fetch-test", metric2["archive_policy/name"])

        # MEASURES ADD
        self.gnocchi('measures',
                     params=("add %s "
                             "-m '2015-03-06T14:33:57Z@43.11' "
                             "--measure '2015-03-06T14:34:12Z@12' ")
                     % metric1["id"], has_output=False)
        self.gnocchi('measures',
                     params=("add %s "
                             "-m '2015-03-06T14:33:57Z@43.11' "
                             "--measure '2015-03-06T14:34:12Z@12' ")
                     % metric2["id"], has_output=False)

        # MEASURES GET with refresh
        self.gnocchi('measures', params=("show %s "
                                         "--aggregation mean "
                                         "--granularity 1 "
                                         "--refresh") % metric1["id"])
        self.gnocchi('measures', params=("show %s "
                                         "--aggregation mean "
                                         "--granularity 1 "
                                         "--refresh") % metric2["id"])

        # MEASURES AGGREGATION METRIC IDS
        result = self.gnocchi(
            'aggregates', params=("'(+ 2 (metric (%s mean) (%s mean)))' "
                                  "--granularity 1 "
                                  "--start 2015-03-06T14:32:00Z "
                                  "--stop 2015-03-06T14:36:00Z "
                                  ) % (metric1["id"], metric2["id"]))
        measures = json.loads(result)
        self.assertEqual(4, len(measures))
        self.assertIn({'granularity': 1.0,
                       'name': '%s/mean' % metric1["id"],
                       'timestamp': '2015-03-06T14:33:57+00:00',
                       'value': 45.11}, measures)
        self.assertIn({'granularity': 1.0,
                       'name': '%s/mean' % metric1["id"],
                       'timestamp': '2015-03-06T14:34:12+00:00',
                       'value': 14.0}, measures)
        self.assertIn({'granularity': 1.0,
                       'name': '%s/mean' % metric2["id"],
                       'timestamp': '2015-03-06T14:33:57+00:00',
                       'value': 45.11}, measures)
        self.assertIn({'granularity': 1.0,
                       'name': '%s/mean' % metric2["id"],
                       'timestamp': '2015-03-06T14:34:12+00:00',
                       'value': 14.0}, measures)

        # MEASURES AGGREGATION METRIC NAMES
        result = self.gnocchi(
            'aggregates', params=(
                "'(+ 2 (metric metric-name mean))' "
                "'original_resource_id like \"metric-res%\"' "
                "--groupby project_id "
                "--groupby user_id "
                "--resource-type generic "
                "--granularity 1 "
                "--start 2015-03-06T14:32:00Z "
                "--stop 2015-03-06T14:36:00Z "
            ))
        measures = json.loads(result)
        self.assertEqual(4, len(measures))
        self.assertIn({'granularity': 1.0,
                       'group': u'project_id: None, user_id: None',
                       'name': u'%s/metric-name/mean' % r1["id"],
                       'timestamp': u'2015-03-06T14:33:57+00:00',
                       'value': 45.11}, measures)
        self.assertIn({'granularity': 1.0,
                       'group': u'project_id: None, user_id: None',
                       'name': u'%s/metric-name/mean' % r1["id"],
                       'timestamp': u'2015-03-06T14:34:12+00:00',
                       'value': 14.0}, measures)
        self.assertIn({'granularity': 1.0,
                       'group': u'project_id: None, user_id: None',
                       'name': u'%s/metric-name/mean' % r2["id"],
                       'timestamp': u'2015-03-06T14:33:57+00:00',
                       'value': 45.11}, measures)
        self.assertIn({'granularity': 1.0,
                       'group': u'project_id: None, user_id: None',
                       'name': u'%s/metric-name/mean' % r2["id"],
                       'timestamp': u'2015-03-06T14:34:12+00:00',
                       'value': 14.0}, measures)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import uuid

from gnocchiclient.tests.functional import base


class ResourceClientTest(base.ClientTestBase):
    RESOURCE_ID = str(uuid.uuid4())
    RESOURCE_ID2 = "foo"
    PROJECT_ID = str(uuid.uuid4())

    def test_help(self):
        self.gnocchi("help", params="resource list")
        self.gnocchi("help", params="resource history")
        self.gnocchi("help", params="resource search")

    def test_resource_scenario(self):
        apname = str(uuid.uuid4())
        # Create an archive policy
        self.gnocchi(
            u'archive-policy', params=u"create %s"
            u" -d granularity:1s,points:86400" % apname)

        # CREATE
        result = self.gnocchi(
            u'resource', params=u"create %s --type generic" %
            self.RESOURCE_ID)
        resource = json.loads(result)
        self.assertEqual(self.RESOURCE_ID, resource["id"])
        self.assertIsNone(resource["project_id"])
        self.assertIsNotNone(resource["started_at"])

        # CREATE FAIL
        result = self.gnocchi('resource',
                              params="create generic -a id:%s" %
                              self.RESOURCE_ID,
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Resource %s already exists (HTTP 409)\n" % self.RESOURCE_ID,
            result)

        # UPDATE
        result = self.gnocchi(
            'resource', params=("update -t generic %s -a project_id:%s "
                                "-n temperature:%s" %
                                (self.RESOURCE_ID, self.PROJECT_ID, apname)))
        resource_updated = json.loads(result)
        self.assertEqual(self.RESOURCE_ID, resource_updated["id"])
        self.assertEqual(self.PROJECT_ID, resource_updated["project_id"])
        self.assertEqual(resource["started_at"],
                         resource_updated["started_at"])
        self.assertIn("temperature", resource_updated["metrics"])

        # GET
        result = self.gnocchi(
            'resource', params="show -t generic %s" % self.RESOURCE_ID)
        resource_got = json.loads(result)
        self.assertEqual(self.RESOURCE_ID, resource_got["id"])
        self.assertEqual(self.PROJECT_ID, resource_got["project_id"])
        self.assertEqual(resource["started_at"], resource_got["started_at"])
        self.assertIn("temperature", resource_got["metrics"])

        # HISTORY
        result = self.gnocchi(
            'resource', params="history --type generic %s" % self.RESOURCE_ID)
        resource_history = json.loads(result)
        self.assertEqual(2, len(resource_history))
        self.assertEqual(self.RESOURCE_ID, resource_history[0]["id"])
        self.assertEqual(self.RESOURCE_ID, resource_history[1]["id"])
        self.assertIsNone(resource_history[0]["project_id"])
        self.assertEqual(self.PROJECT_ID, resource_history[1]["project_id"])

        # LIST
        result = self.gnocchi('resource', params="list -t generic")
        self.assertIn(self.RESOURCE_ID,
                      [r['id'] for r in json.loads(result)])
        resource_list = [r for r in json.loads(result)
                         if r['id'] == self.RESOURCE_ID][0]
        self.assertEqual(self.RESOURCE_ID, resource_list["id"])
        self.assertEqual(self.PROJECT_ID, resource_list["project_id"])
        self.assertEqual(resource["started_at"], resource_list["started_at"])

        # Search
        result = self.gnocchi('resource',
                              params=("search --type generic "
                                      "'project_id=%s'"
                                      ) % self.PROJECT_ID)
        resource_list = json.loads(result)[0]
        self.assertEqual(self.RESOURCE_ID, resource_list["id"])
        self.assertEqual(self.PROJECT_ID, resource_list["project_id"])
        self.assertEqual(resource["started_at"], resource_list["started_at"])

        # UPDATE with Delete metric
        result = self.gnocchi(
            'resource', params=("update -t generic %s -a project_id:%s "
                                "-d temperature" %
                                (self.RESOURCE_ID, self.PROJECT_ID)))
        resource_updated = json.loads(result)
        self.assertNotIn("temperature", resource_updated["metrics"])

        result = self.gnocchi(
            'resource', params=("update %s -d temperature" % self.RESOURCE_ID),
            fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Metric name temperature not found\n",
            result)

        # CREATE 2
        result = self.gnocchi(
            'resource', params=("create %s -t generic "
                                "-a project_id:%s"
                                ) % (self.RESOURCE_ID2, self.PROJECT_ID))
        resource2 = json.loads(result)
        self.assertEqual(self.RESOURCE_ID2,
                         resource2["original_resource_id"])
        self.assertEqual(self.PROJECT_ID, resource2["project_id"])
        self.assertIsNotNone(resource2["started_at"])

        # Search + limit + short
        result = self.gnocchi('resource',
                              params=("search "
                                      "-t generic "
                                      "'project_id=%s' "
                                      "--sort started_at:asc "
                                      "--marker %s "
                                      "--limit 1"
                                      ) % (self.PROJECT_ID, self.RESOURCE_ID))
        resource_limit = json.loads(result)[0]
        self.assertEqual(self.RESOURCE_ID2,
                         resource_limit["original_resource_id"])
        self.assertEqual(self.PROJECT_ID, resource_limit["project_id"])
        self.assertEqual(resource2["started_at"], resource_limit["started_at"])

        # DELETE
        self.gnocchi('resource',
                     params="delete %s" % self.RESOURCE_ID,
                     has_output=False)
        self.gnocchi('resource',
                     params="delete %s" % self.RESOURCE_ID2,
                     has_output=False)

        # GET FAIL
        result = self.gnocchi('resource',
                              params="show --type generic %s" %
                              self.RESOURCE_ID,
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Resource %s does not exist (HTTP 404)\n" % self.RESOURCE_ID,
            result)

        # DELETE FAIL
        result = self.gnocchi('resource',
                              params="delete %s" % self.RESOURCE_ID,
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Resource %s does not exist (HTTP 404)\n" % self.RESOURCE_ID,
            result)

        # Create and Batch Delete
        result1 = self.gnocchi(
            u'resource', params=u"create %s --type generic" %
            self.RESOURCE_ID)
        result2 = self.gnocchi(
            u'resource', params=u"create %s --type generic" %
            self.RESOURCE_ID2)
        resource1 = json.loads(result1)
        resource2 = json.loads(result2)
        self.assertEqual(self.RESOURCE_ID, resource1['id'])
        self.assertEqual(self.RESOURCE_ID2, resource2['original_resource_id'])
        result3 = self.gnocchi(
            'resource batch delete ',
            params=("'id in [%s, %s]' "
                    "-t generic") % (resource1["id"], resource2["id"]))
        resource3 = json.loads(result3)
        self.assertEqual(2, int(resource3["deleted"]))
        result4 = self.gnocchi(
            'resource batch delete ',
            params=("'id in [%s, %s]' "
                    "-t generic") % (resource1["id"], resource2["id"]))
        resource4 = json.loads(result4)
        self.assertEqual(0, int(resource4["deleted"]))

        # LIST EMPTY
        result = self.gnocchi('resource', params="list -t generic")
        resource_ids = [r['id'] for r in json.loads(result)]
        self.assertNotIn(self.RESOURCE_ID, resource_ids)
        self.assertNotIn(self.RESOURCE_ID2, resource_ids)



# -*- encoding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import unittest

from requests import models

from gnocchiclient import exceptions


class ExceptionsTest(unittest.TestCase):
    def test_from_response_404(self):
        r = models.Response()
        r.status_code = 404
        r.headers['Content-Type'] = "application/json"
        r._content = json.dumps(
            {"description": "Archive policy rule foobar does not exist"}
        ).encode('utf-8')
        exc = exceptions.from_response(r)
        self.assertIsInstance(exc, exceptions.ArchivePolicyRuleNotFound)

    def test_from_response_404_with_detail(self):
        r = models.Response()
        r.status_code = 404
        r.headers['Content-Type'] = "application/json"
        r._content = json.dumps({
            "code": 404,
            "description": {
                "cause": "Aggregation method does not exist for this metric",
                "detail": {
                    "aggregation_method": "rate:mean",
                    "metric": "a914dad6-b8f6-42f6-b090-6daa29725caf",
                }},
            "title": "Not Found"
        }).encode('utf-8')
        exc = exceptions.from_response(r)
        self.assertIsInstance(exc, exceptions.ClientException)

    def test_resource_type_before_resource(self):
        r = models.Response()
        r.status_code = 404
        r.headers['Content-Type'] = "application/json"
        r._content = json.dumps(
            {"description": "Resource type foobar does not exist"}
        ).encode('utf-8')
        exc = exceptions.from_response(r)
        self.assertIsInstance(exc, exceptions.ResourceTypeNotFound)

    def test_from_response_keystone_401(self):
        r = models.Response()
        r.status_code = 401
        r.headers['Content-Type'] = "application/json"
        r._content = json.dumps({"error": {
            "message": "The request you have made requires authentication.",
            "code": 401, "title": "Unauthorized"}}
        ).encode('utf-8')
        exc = exceptions.from_response(r)
        self.assertIsInstance(exc, exceptions.Unauthorized)
        self.assertEqual("The request you have made requires authentication.",
                         exc.message)

    def test_from_response_unknown_middleware(self):
        r = models.Response()
        r.status_code = 400
        r.headers['Content-Type'] = "application/json"
        r._content = json.dumps(
            {"unknown": "random message"}
        ).encode('utf-8')
        exc = exceptions.from_response(r)
        self.assertIsInstance(exc, exceptions.ClientException)
        self.assertEqual('{"unknown": "random message"}', exc.message)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import uuid

from gnocchiclient.tests.functional import base


class ArchivePolicyRuleClientTest(base.ClientTestBase):
    def test_archive_policy_rule_scenario(self):
        apname = str(uuid.uuid4())
        # Create an archive policy
        self.gnocchi(
            u'archive-policy', params=u"create %s"
            u" -d granularity:1s,points:86400" % apname)

        # CREATE
        result = self.gnocchi(
            u'archive-policy-rule', params=u"create test"
            u" --archive-policy-name %s"
            u" --metric-pattern 'disk.io.*'" % apname)
        policy_rule = json.loads(result)
        self.assertEqual('test', policy_rule["name"])

        # CREATE FAIL
        result = self.gnocchi(
            u'archive-policy-rule', params=u"create test"
            u" --archive-policy-name high"
            u" --metric-pattern 'disk.io.*'",
            fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Archive policy rule test already exists (HTTP 409)\n",
            result)
        # GET
        result = self.gnocchi(
            'archive-policy-rule', params="show test")
        policy_rule = json.loads(result)
        self.assertEqual("test", policy_rule["name"])

        # LIST
        result = self.gnocchi('archive-policy-rule', params="list")
        rules = json.loads(result)
        rule_from_list = [p for p in rules
                          if p['name'] == 'test'][0]
        for field in ["metric_pattern", "archive_policy_name"]:
            self.assertEqual(policy_rule[field], rule_from_list[field])

        # CREATE - RENAME - GET
        self.gnocchi(
            u'archive-policy-rule', params=u"create to_rename"
            u" --archive-policy-name %s"
            u" --metric-pattern 'disk.io.*'" % apname)
        result = self.gnocchi(
            u'archive-policy-rule', params=u'update to_rename'
            u' --name renamed')
        policy_rule = json.loads(result)
        self.assertEqual('renamed', policy_rule["name"])

        result = self.gnocchi(
            'archive-policy-rule', params="show renamed")
        policy_rule = json.loads(result)
        self.assertEqual("renamed", policy_rule["name"])

        # CONFIRM RENAME
        result = self.gnocchi('archive-policy-rule',
                              params="show to_rename",
                              fail_ok=True, merge_stderr=True)
        self.assertEqual(
            "Archive policy rule to_rename does not exist (HTTP 404)\n",
            result)

        # RENAME FAIL
        result = self.gnocchi(
            'archive-policy-rule', params='update test'
            ' --name renamed',
            fail_ok=True, merge_stderr=True)
        self.assertEqual(
            'Archive policy rule test does not support change:'
            ' Archive policy rule renamed already exists. (HTTP 400)\n',
            result)

        # DELETE
        self.gnocchi('archive-policy-rule', params="delete test",
                     has_output=False)

        # GET FAIL
        result = self.gnocchi('archive-policy-rule',
                              params="show test",
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Archive policy rule test does not exist (HTTP 404)\n",
            result)

        # DELETE FAIL
        result = self.gnocchi('archive-policy-rule',
                              params="delete test",
                              fail_ok=True, merge_stderr=True,
                              has_output=False)
        self.assertEqual(
            "Archive policy rule test does not exist (HTTP 404)\n",
            result)
